@@ -33,6 +33,7 @@
 
 // Local Includes
 #include "adblocknetworkreply.h"
+#include "adblockwidget.h"
 #include "webpage.h"
 
 // KDE Includes
@@ -46,11 +47,12 @@
 
 
 AdBlockManager::AdBlockManager(QObject *parent)
-        : QObject(parent)
-        , _isAdblockEnabled(false)
-        , _isHideAdsEnabled(false)
-        , _index(0)
+    : QObject(parent)
+    , _isAdblockEnabled(false)
+    , _isHideAdsEnabled(false)
+    , _index(0)
 {
+    loadSettings();
 }
 
 
@@ -106,7 +108,7 @@ void AdBlockManager::loadSettings(bool checkUpdateDate)
 
     // else
     QStringList titles = ReKonfig::subscriptionTitles();
-    foreach(const QString &title, titles)
+    foreach(const QString & title, titles)
     {
         rules = rulesGroup.readEntry(title + "-rules" , QStringList());
         loadRules(rules);
@@ -116,7 +118,7 @@ void AdBlockManager::loadSettings(bool checkUpdateDate)
 
 void AdBlockManager::loadRules(const QStringList &rules)
 {
-    foreach(const QString &stringRule, rules)
+    foreach(const QString & stringRule, rules)
     {
         // ! rules are comments
         if (stringRule.startsWith('!'))
@@ -179,7 +181,6 @@ QNetworkReply *AdBlockManager::block(const QNetworkRequest &request, WebPage *pa
     const QString host = request.url().host();
 
     // check white rules before :)
-
     if (_hostWhiteList.match(host))
     {
         kDebug() << "****ADBLOCK: WHITE RULE (@@) Matched by host matcher: ***********";
@@ -187,7 +188,7 @@ QNetworkReply *AdBlockManager::block(const QNetworkRequest &request, WebPage *pa
         return 0;
     }
 
-    foreach(const AdBlockRule &filter, _whiteList)
+    foreach(const AdBlockRule & filter, _whiteList)
     {
         if (filter.match(request, urlString, urlStringLowerCase))
         {
@@ -206,7 +207,11 @@ QNetworkReply *AdBlockManager::block(const QNetworkRequest &request, WebPage *pa
         return reply;
     }
 
-    foreach(const AdBlockRule &filter, _blackList)
+    // get sure page is extant to perform elements check. bug:282012
+    if (!page)
+        return 0;
+
+    foreach(const AdBlockRule & filter, _blackList)
     {
         if (filter.match(request, urlString, urlStringLowerCase))
         {
@@ -251,7 +256,7 @@ void AdBlockManager::applyHidingRules(WebPage *page)
     QWebElement document = page->mainFrame()->documentElement();
 
     // HIDE RULES
-    foreach(const QString &filter, _hideList)
+    foreach(const QString & filter, _hideList)
     {
         QWebElementCollection elements = document.findAll(filter);
 
@@ -274,7 +279,6 @@ void AdBlockManager::updateNextSubscription()
     if (_index < locations.size())
     {
         QString urlString = locations.at(_index);
-        kDebug() << "DOWNLOADING FROM " << urlString;
         KUrl subUrl = KUrl(urlString);
 
         KIO::TransferJob* job = KIO::get(subUrl , KIO::Reload , KIO::HideProgressInfo);
@@ -300,10 +304,9 @@ void AdBlockManager::slotResult(KJob *job)
     if (job->error())
         return;
 
-    kDebug() << "SAVING RULES..";
     QList<QByteArray> list = _buffer.split('\n');
     QStringList ruleList;
-    foreach(const QByteArray &ba, list)
+    foreach(const QByteArray & ba, list)
     {
         ruleList << QString(ba);
     }
@@ -333,7 +336,7 @@ void AdBlockManager::subscriptionData(KIO::Job* job, const QByteArray& data)
 void AdBlockManager::saveRules(const QStringList &rules)
 {
     QStringList cleanedRules;
-    foreach(const QString &r, rules)
+    foreach(const QString & r, rules)
     {
         if (!r.startsWith('!') && !r.startsWith('[') && !r.isEmpty())
             cleanedRules << r;
@@ -364,3 +367,19 @@ void AdBlockManager::addSubscription(const QString &title, const QString &locati
     ReKonfig::setSubscriptionTitles(titles);
     ReKonfig::setSubscriptionLocations(locations);
 }
+
+
+void AdBlockManager::showSettings()
+{
+    QPointer<KDialog> dialog = new KDialog();
+    dialog->setCaption(i18nc("@title:window", "Ad Block Settings"));
+    dialog->setButtons(KDialog::Ok | KDialog::Cancel);
+
+    AdBlockWidget widget;
+    dialog->setMainWidget(&widget);
+    connect(dialog, SIGNAL(okClicked()), &widget, SLOT(save()));
+    connect(dialog, SIGNAL(okClicked()), this, SLOT(loadSettings()));
+    dialog->exec();
+
+    dialog->deleteLater();
+}
