@@ -36,12 +36,12 @@
 // Local Includes
 #include "adblockmanager.h"
 #include "bookmarkprovider.h"
+#include "downloadmanager.h"
 #include "filterurljob.h"
 #include "historymanager.h"
 #include "iconmanager.h"
 #include "mainview.h"
 #include "mainwindow.h"
-#include "messagebar.h"
 #include "opensearchmanager.h"
 #include "sessionmanager.h"
 #include "stackedurlbar.h"
@@ -58,8 +58,10 @@
 #include <ThreadWeaver/Weaver>
 #include <KAction>
 #include <KWindowSystem>
+#include <KWindowInfo>
 #include <KGlobal>
 #include <KCharsets>
+#include <KMessageBox>
 
 // Qt Includes
 #include <QVBoxLayout>
@@ -69,8 +71,8 @@ using namespace ThreadWeaver;
 
 
 Application::Application()
-        : KUniqueApplication()
-        , _privateBrowsingAction(0)
+    : KUniqueApplication()
+    , _privateBrowsingAction(0)
 {
     connect(Weaver::instance(), SIGNAL(jobDone(ThreadWeaver::Job*)),
             this, SLOT(loadResolvedUrl(ThreadWeaver::Job*)));
@@ -93,6 +95,52 @@ Application::~Application()
         delete window.data();
         window.clear();
     }
+
+    if (!m_historyManager.isNull())
+    {
+        delete m_historyManager.data();
+        m_historyManager.clear();
+    }
+
+    if (!m_bookmarkProvider.isNull())
+    {
+        delete m_bookmarkProvider.data();
+        m_bookmarkProvider.clear();
+    }
+
+    if (!m_sessionManager.isNull())
+    {
+        delete m_sessionManager.data();
+        m_sessionManager.clear();
+    }
+
+    if (!m_opensearchManager.isNull())
+    {
+        delete m_opensearchManager.data();
+        m_opensearchManager.clear();
+    }
+
+    if (!m_iconManager.isNull())
+    {
+        delete m_iconManager.data();
+        m_iconManager.clear();
+    }
+
+    if (!m_adblockManager.isNull())
+    {
+        delete m_adblockManager.data();
+        m_adblockManager.clear();
+    }
+
+    // TODO:
+    // add a check to NOT close rekonq
+    // until last download is finished
+
+    if (!m_downloadManager.isNull())
+    {
+        delete m_adblockManager.data();
+        m_adblockManager.clear();
+    }
 }
 
 
@@ -122,14 +170,7 @@ int Application::newInstance()
     if (isRekonqCrashed && isFirstLoad)
     {
         loadUrl(KUrl("about:closedTabs"), Rekonq::NewWindow);
-        MessageBar *msgBar = new MessageBar(i18n("It seems rekonq was not closed properly. Do you want "
-                                            "to restore the last saved session?")
-                                            , mainWindow()->currentTab()
-                                            , QMessageBox::Warning
-                                            , MessageBar::Yes | MessageBar::No);
-
-        connect(msgBar, SIGNAL(accepted()), sessionManager(), SLOT(restoreSession()));
-        mainWindow()->currentTab()->insertBar(msgBar);
+        mainWindow()->currentTab()->showMessageBar();
     }
 
     if (areThereArguments)
@@ -182,9 +223,10 @@ int Application::newInstance()
                 loadUrl(KUrl("about:home"), Rekonq::NewWindow);
                 break;
             case 2: // restore session
-                sessionManager()->restoreSession();
-                kDebug() << "session restored following settings";
-                break;
+                if (sessionManager()->restoreSession())
+                {
+                    break;
+                }
             default:
                 newMainWindow()->homePage();
                 break;
@@ -197,12 +239,10 @@ int Application::newInstance()
             case 0: // new tab page
                 loadUrl(KUrl("about:home") , Rekonq::NewWindow);
                 break;
-            case 1: // blank page
-                loadUrl(KUrl("about:blank") , Rekonq::NewWindow);
-                break;
             case 2: // homepage
                 loadUrl(KUrl(ReKonfig::homePage()) , Rekonq::NewWindow);
                 break;
+            case 1: // blank page
             default:
                 loadUrl(KUrl("about:blank") , Rekonq::NewWindow);
                 break;
@@ -259,9 +299,14 @@ MainWindow *Application::mainWindow()
 
     if (!active)
     {
-        if(m_mainWindows.isEmpty())
+        if (m_mainWindows.isEmpty())
             return 0;
 
+        Q_FOREACH(const QWeakPointer<MainWindow> &pointer, m_mainWindows)
+        {
+            if (KWindowInfo(pointer.data()->effectiveWinId(), NET::WMDesktop, 0).isOnCurrentDesktop())
+                return pointer.data();
+        }
         return m_mainWindows.at(0).data();
     }
     return active;
@@ -272,7 +317,7 @@ HistoryManager *Application::historyManager()
 {
     if (m_historyManager.isNull())
     {
-        m_historyManager = new HistoryManager(instance());
+        m_historyManager = new HistoryManager;
     }
     return m_historyManager.data();
 }
@@ -282,7 +327,7 @@ BookmarkProvider *Application::bookmarkProvider()
 {
     if (m_bookmarkProvider.isNull())
     {
-        m_bookmarkProvider = new BookmarkProvider(instance());
+        m_bookmarkProvider = new BookmarkProvider;
     }
     return m_bookmarkProvider.data();
 }
@@ -292,7 +337,7 @@ SessionManager *Application::sessionManager()
 {
     if (m_sessionManager.isNull())
     {
-        m_sessionManager = new SessionManager(instance());
+        m_sessionManager = new SessionManager;
     }
     return m_sessionManager.data();
 }
@@ -302,7 +347,7 @@ OpenSearchManager *Application::opensearchManager()
 {
     if (m_opensearchManager.isNull())
     {
-        m_opensearchManager = new OpenSearchManager(instance());
+        m_opensearchManager = new OpenSearchManager;
     }
     return m_opensearchManager.data();
 }
@@ -312,12 +357,22 @@ IconManager *Application::iconManager()
 {
     if (m_iconManager.isNull())
     {
-        m_iconManager = new IconManager(instance());
+        m_iconManager = new IconManager;
     }
     return m_iconManager.data();
 }
 
 
+DownloadManager *Application::downloadManager()
+{
+    if (m_downloadManager.isNull())
+    {
+        m_downloadManager = new DownloadManager(instance());
+    }
+    return m_downloadManager.data();
+}
+
+
 void Application::loadUrl(const KUrl& url, const Rekonq::OpenType& type)
 {
     if (url.isEmpty())
@@ -380,6 +435,8 @@ void Application::loadUrl(const KUrl& url, const Rekonq::OpenType& type)
 MainWindow *Application::newMainWindow(bool withTab)
 {
     MainWindow *w = new MainWindow();
+    // This is used to track which window was activated most recently
+    w->installEventFilter(this);
 
     if (withTab)
         w->mainView()->newWebTab();    // remember using newWebTab and NOT newTab here!!
@@ -407,7 +464,7 @@ AdBlockManager *Application::adblockManager()
 {
     if (m_adblockManager.isNull())
     {
-        m_adblockManager = new AdBlockManager(instance());
+        m_adblockManager = new AdBlockManager;
     }
     return m_adblockManager.data();
 }
@@ -436,6 +493,28 @@ void Application::newWindow()
 }
 
 
+bool Application::eventFilter(QObject* watched, QEvent* event)
+{
+    // Track which window was activated most recently to prefer it on window choosing
+    // (e.g. when another application opens a link)
+    if (event->type() == QEvent::WindowActivate)
+    {
+        MainWindow *window = qobject_cast<MainWindow*>(watched);
+        if (window)
+        {
+            if (m_mainWindows.at(0).data() != window)
+            {
+                int index = m_mainWindows.indexOf(QWeakPointer<MainWindow>(window));
+                Q_ASSERT(index != -1);
+                m_mainWindows.prepend(m_mainWindows.takeAt(index));
+            }
+        }
+    }
+
+    return QObject::eventFilter(watched, event);
+}
+
+
 void Application::updateConfiguration()
 {
     // ============== Tabs ==================
@@ -445,7 +524,7 @@ void Application::updateConfiguration()
         MainView *mv = w.data()->mainView();
         mv->updateTabBar();
 
-        mv->tabBar()->setAnimatedTabHighlighting( ReKonfig::animatedTabHighlighting() );
+        mv->tabBar()->setAnimatedTabHighlighting(ReKonfig::animatedTabHighlighting());
 
         if (b)
             mv->tabBar()->setSelectionBehaviorOnRemove(QTabBar::SelectPreviousTab);
@@ -466,18 +545,19 @@ void Application::updateConfiguration()
     // compute font size
     // (I have to admit I know nothing about these DPI questions..: copied from kwebkitpart, as someone suggested)
     // font size in pixels =  font size in inches × screen dpi
-    int defaultFontSize = ReKonfig::defaultFontSize();
-    int minimumFontSize = ReKonfig::minFontSize();
+    if (mainWindow() && mainWindow()->currentTab())
+    {
+        int logDpiY = mainWindow()->currentTab()->view()->logicalDpiY();
+        float toPix = (logDpiY < 96.0)
+                      ? 96.0 / 72.0
+                      : logDpiY / 72.0 ;
 
-    int logDpiY = mainWindow()->currentTab()->view()->logicalDpiY();
-    kDebug() << "Logical Dot per Inch Y: " << logDpiY;
+        int defaultFontSize = ReKonfig::defaultFontSize();
+        int minimumFontSize = ReKonfig::minFontSize();
 
-    float toPix = (logDpiY < 96.0)
-                  ? 96.0 / 72.0
-                  : logDpiY / 72.0 ;
-
-    defaultSettings->setFontSize(QWebSettings::DefaultFontSize, qRound(defaultFontSize * toPix));
-    defaultSettings->setFontSize(QWebSettings::MinimumFontSize, qRound(minimumFontSize * toPix));
+        defaultSettings->setFontSize(QWebSettings::DefaultFontSize, qRound(defaultFontSize * toPix));
+        defaultSettings->setFontSize(QWebSettings::MinimumFontSize, qRound(minimumFontSize * toPix));
+    }
 
     // encodings
     QString enc = ReKonfig::defaultEncoding();
@@ -514,14 +594,20 @@ void Application::updateConfiguration()
         QWebSettings::setOfflineStorageDefaultQuota(50000);
     }
 
-    // Applies user defined CSS to all open webpages. If there no longer is a
-    // user defined CSS removes it from all open webpages.
-    if (!ReKonfig::userCSS().isEmpty())
-        defaultSettings->setUserStyleSheetUrl(ReKonfig::userCSS());
+    // Applies user/system defined CSS to all open webpages.
+    ReKonfig::userCSS().isEmpty()
+    ? defaultSettings->setUserStyleSheetUrl(KUrl(KStandardDirs::locate("appdata" , "default_rekonq.css")))
+    : defaultSettings->setUserStyleSheetUrl(ReKonfig::userCSS())
+    ;
 
     // ====== load Settings on main classes
     historyManager()->loadSettings();
-    adblockManager()->loadSettings();
+
+    defaultSettings = 0;
+
+    if (!mainWindow())
+        return;
+
     if (!ReKonfig::useFavicon())
         mainWindow()->setWindowIcon(KIcon("rekonq"));
     else
@@ -553,66 +639,10 @@ void Application::updateConfiguration()
         break;
 
     default: // non extant case
+        ASSERT_NOT_REACHED();
         break;
     }
 
-    defaultSettings = 0;
-}
-
-
-void Application::addDownload(const QString &srcUrl, const QString &destUrl)
-{
-    QWebSettings *globalSettings = QWebSettings::globalSettings();
-    if (globalSettings->testAttribute(QWebSettings::PrivateBrowsingEnabled))
-        return;
-    QString downloadFilePath = KStandardDirs::locateLocal("appdata" , "downloads");
-    QFile downloadFile(downloadFilePath);
-    if (!downloadFile.open(QFile::WriteOnly | QFile::Append))
-    {
-        kDebug() << "Unable to open download file (WRITE mode)..";
-        return;
-    }
-    QDataStream out(&downloadFile);
-    out << srcUrl;
-    out << destUrl;
-    out << QDateTime::currentDateTime();
-    downloadFile.close();
-}
-
-
-DownloadList Application::downloads()
-{
-    DownloadList list;
-
-    QString downloadFilePath = KStandardDirs::locateLocal("appdata" , "downloads");
-    QFile downloadFile(downloadFilePath);
-    if (!downloadFile.open(QFile::ReadOnly))
-    {
-        kDebug() << "Unable to open download file (READ mode)..";
-        return list;
-    }
-
-    QDataStream in(&downloadFile);
-    while (!in.atEnd())
-    {
-        QString srcUrl;
-        in >> srcUrl;
-        QString destUrl;
-        in >> destUrl;
-        QDateTime dt;
-        in >> dt;
-        DownloadItem item(srcUrl, destUrl, dt);
-        list << item;
-    }
-    return list;
-}
-
-
-bool Application::clearDownloadsHistory()
-{
-    QString downloadFilePath = KStandardDirs::locateLocal("appdata" , "downloads");
-    QFile downloadFile(downloadFilePath);
-    return downloadFile.remove();
 }
 
 
@@ -666,3 +696,37 @@ void Application::setPrivateBrowsingMode(bool b)
             loadUrl(KUrl("about:home"), Rekonq::NewWindow);
     }
 }
+
+
+void Application::queryQuit()
+{
+    if (mainWindowList().count() > 1)
+    {
+        int answer = KMessageBox::questionYesNoCancel(
+                         mainWindow(),
+                         i18n("Wanna close the window or the whole app?"),
+                         i18n("Application/Window closing..."),
+                         KGuiItem(i18n("C&lose Current Window"),
+                                  KIcon("window-close")),
+                         KStandardGuiItem::quit(),
+                         KStandardGuiItem::cancel(),
+                         "confirmClosingMultipleWindows"
+                     );
+
+        switch (answer)
+        {
+        case KMessageBox::Yes:
+            mainWindow()->close();
+            return;
+
+        case KMessageBox::No:
+            break;
+
+        default:
+            return;
+        }
+    }
+
+    // in case of just one window...
+    quit();
+}
