@@ -33,6 +33,9 @@
 // Rekonq Includes
 #include "rekonq_defines.h"
 
+// Local Includes
+#include "downloaditem.h"
+
 // KDE Includes
 #include <KUniqueApplication>
 
@@ -43,7 +46,7 @@
 // Forward Declarations
 class AdBlockManager;
 class BookmarkProvider;
-class DownloadItem;
+class DownloadManager;
 class HistoryManager;
 class IconManager;
 class MainWindow;
@@ -59,34 +62,15 @@ class Job;
 
 
 typedef QList< QWeakPointer<MainWindow> > MainWindowList;
-typedef QList<DownloadItem> DownloadList;
-
-
-class DownloadItem
-{
-public:
-    DownloadItem() {}
-    explicit DownloadItem(const QString &srcUrl,
-                          const QString &destUrl,
-                          const QDateTime &d
-                         )
-            : srcUrlString(srcUrl)
-            , destUrlString(destUrl)
-            , dateTime(d)
-    {}
-
-    QString srcUrlString;
-    QString destUrlString;
-    QDateTime dateTime;
-};
 
 
 // ---------------------------------------------------------------------------------------------------------------
 
+
 #define rApp Application::instance()
 
 /**
-  *
+  * Rekonq Application class
   */
 class REKONQ_TESTS_EXPORT Application : public KUniqueApplication
 {
@@ -95,6 +79,7 @@ class REKONQ_TESTS_EXPORT Application : public KUniqueApplication
 public:
     Application();
     ~Application();
+
     int newInstance();
     static Application *instance();
 
@@ -108,11 +93,7 @@ public:
     AdBlockManager *adblockManager();
     OpenSearchManager *opensearchManager();
     IconManager *iconManager();
-
-    // DOWNLOADS MANAGEMENT METHODS
-    void addDownload(const QString &srcUrl, const QString &destUrl);
-    DownloadList downloads();
-    bool clearDownloadsHistory();
+    DownloadManager *downloadManager();
 
     KAction *privateBrowsingAction()
     {
@@ -134,6 +115,10 @@ public slots:
     void newWindow();
     void removeMainWindow(MainWindow *window);
 
+protected:
+    // This is used to track which window was activated most recently
+    bool eventFilter(QObject *watched, QEvent *event);
+
 private slots:
     /**
      * Any actions that can be delayed until the window is visible
@@ -147,6 +132,8 @@ private slots:
     // the general place to set private browsing
     void setPrivateBrowsingMode(bool);
 
+    void queryQuit();
+
 private:
     QWeakPointer<HistoryManager> m_historyManager;
     QWeakPointer<BookmarkProvider> m_bookmarkProvider;
@@ -154,6 +141,7 @@ private:
     QWeakPointer<AdBlockManager> m_adblockManager;
     QWeakPointer<OpenSearchManager> m_opensearchManager;
     QWeakPointer<IconManager> m_iconManager;
+    QWeakPointer<DownloadManager> m_downloadManager;
 
     MainWindowList m_mainWindows;
 
