@@ -36,6 +36,7 @@
 #include "application.h"
 #include "bookmarkprovider.h"
 #include "bookmarkowner.h"
+#include "webtab.h"
 
 // Qt Includes
 #include <QtGui/QFrame>
@@ -46,7 +47,7 @@ BookmarkMenu::BookmarkMenu(KBookmarkManager *manager,
                            KBookmarkOwner *owner,
                            KMenu *menu,
                            KActionCollection* actionCollection)
-        : KBookmarkMenu(manager, owner, menu, actionCollection)
+    : KBookmarkMenu(manager, owner, menu, actionCollection)
 {
 }
 
@@ -55,7 +56,7 @@ BookmarkMenu::BookmarkMenu(KBookmarkManager  *manager,
                            KBookmarkOwner  *owner,
                            KMenu  *parentMenu,
                            const QString &parentAddress)
-        : KBookmarkMenu(manager, owner, parentMenu, parentAddress)
+    : KBookmarkMenu(manager, owner, parentMenu, parentAddress)
 {
 }
 
@@ -109,17 +110,13 @@ void BookmarkMenu::refill()
 
     if (isRoot())
     {
-        addAddBookmark();
         addAddBookmarksList();
-        addNewFolder();
         addEditBookmarks();
     }
     else
     {
         addOpenFolderInTabs();
-        addAddBookmark();
         addAddBookmarksList();
-        addNewFolder();
     }
 }
 
@@ -157,12 +154,13 @@ void BookmarkMenu::actionHovered()
 
 
 BookmarkToolBar::BookmarkToolBar(KToolBar *toolBar, QObject *parent)
-        : QObject(parent)
-        , m_toolBar(toolBar)
-        , m_currentMenu(0)
-        , m_dragAction(0)
-        , m_dropAction(0)
-        , m_filled(false)
+    : QObject(parent)
+    , m_toolBar(toolBar)
+    , m_currentMenu(0)
+    , m_dragAction(0)
+    , m_dropAction(0)
+    , m_checkedAction(0)
+    , m_filled(false)
 {
     toolBar->setContextMenuPolicy(Qt::CustomContextMenu);
     connect(toolBar, SIGNAL(customContextMenuRequested(const QPoint &)), this, SLOT(contextMenu(const QPoint &)));
@@ -188,13 +186,18 @@ KToolBar* BookmarkToolBar::toolBar()
 void BookmarkToolBar::contextMenu(const QPoint &point)
 {
     KBookmarkActionInterface *action = dynamic_cast<KBookmarkActionInterface*>(toolBar()->actionAt(point));
-    KBookmark bookmark;
+    KBookmark bookmark = rApp->bookmarkProvider()->bookmarkManager()->toolbar();
+    bool nullAction = true;
     if (action)
+    {
         bookmark = action->bookmark();
+        nullAction = false;
+    }
 
     BookmarksContextMenu menu(bookmark,
                               rApp->bookmarkProvider()->bookmarkManager(),
-                              rApp->bookmarkProvider()->bookmarkOwner());
+                              rApp->bookmarkProvider()->bookmarkOwner(),
+                              nullAction);
     menu.exec(toolBar()->mapToGlobal(point));
 }
 
@@ -222,76 +225,117 @@ void BookmarkToolBar::hideMenu()
 
 bool BookmarkToolBar::eventFilter(QObject *watched, QEvent *event)
 {
-    if (m_currentMenu && m_currentMenu->isVisible() && !m_currentMenu->rect().contains(m_currentMenu->mapFromGlobal(QCursor::pos())))
+    if (m_currentMenu && m_currentMenu->isVisible()
+            && !m_currentMenu->rect().contains(m_currentMenu->mapFromGlobal(QCursor::pos())))
     {
         // To switch root folders as in a menubar
+
         KBookmarkActionMenu* act = dynamic_cast<KBookmarkActionMenu *>(toolBar()->actionAt(toolBar()->mapFromGlobal(QCursor::pos())));
-        if (event->type() == QEvent::MouseMove && act && m_currentMenu && act->menu() != m_currentMenu)
+
+        if (event->type() == QEvent::MouseMove && act && act->menu() != m_currentMenu)
         {
             m_currentMenu->hide();
             QPoint pos = toolBar()->mapToGlobal(toolBar()->widgetForAction(act)->pos());
             act->menu()->popup(QPoint(pos.x(), pos.y() + toolBar()->widgetForAction(act)->height()));
         }
+        else if (event->type() == QEvent::MouseButtonPress && act)
+        {
+            m_currentMenu->hide();
+        }
+
+        return QObject::eventFilter(watched, event);
     }
-    else if (watched == toolBar())
+
+    switch (event->type())
     {
-        if (event->type() == QEvent::Show)
+    case QEvent::Show:
+    {
+        if (!m_filled)
         {
-            if (!m_filled)
+            rApp->bookmarkProvider()->fillBookmarkBar(this);
+            m_filled = true;
+        }
+    }
+    break;
+
+    case QEvent::ActionRemoved:
+    {
+        QActionEvent *actionEvent = static_cast<QActionEvent*>(event);
+        if (actionEvent && actionEvent->action() != m_dropAction)
+        {
+            QWidget *widget = toolBar()->widgetForAction(actionEvent->action());
+            if (widget)
             {
-                rApp->bookmarkProvider()->fillBookmarkBar(this);
-                m_filled = true;
+                widget->removeEventFilter(this);
             }
         }
-        if (event->type() == QEvent::ActionRemoved)
+    }
+    break;
+
+    case QEvent::ParentChange:
+    {
+        QActionEvent *actionEvent = static_cast<QActionEvent*>(event);
+        if (actionEvent && actionEvent->action() != m_dropAction)
         {
-            QActionEvent *actionEvent = static_cast<QActionEvent*>(event);
-            if (actionEvent && actionEvent->action() != m_dropAction)
+            QWidget *widget = toolBar()->widgetForAction(actionEvent->action());
+            if (widget)
             {
-                QWidget *widget = toolBar()->widgetForAction(actionEvent->action());
-                if (widget)
+                widget->removeEventFilter(this);
+            }
+        }
+    }
+    break;
+
+    case QEvent::DragEnter:
+    {
+        QDragEnterEvent *dragEvent = static_cast<QDragEnterEvent*>(event);
+        if (dragEvent->mimeData()->hasFormat("application/rekonq-bookmark") || dragEvent->mimeData()->hasFormat("text/uri-list") || dragEvent->mimeData()->hasFormat("text/plain"))
+        {
+            QFrame* dropIndicatorWidget = new QFrame(toolBar());
+            dropIndicatorWidget->setFrameShape(QFrame::VLine);
+            m_dropAction = toolBar()->insertWidget(toolBar()->actionAt(dragEvent->pos()), dropIndicatorWidget);
+
+            dragEvent->accept();
+        }
+    }
+    break;
+
+    case QEvent::DragLeave:
+    {
+        QDragLeaveEvent *dragEvent = static_cast<QDragLeaveEvent*>(event);
+
+        if (m_checkedAction)
+        {
+            m_checkedAction->setCheckable(false);
+            m_checkedAction->setChecked(false);
+        }
+
+        delete m_dropAction;
+        m_dropAction = 0;
+        dragEvent->accept();
+    }
+    break;
+
+    case QEvent::DragMove:
+    {
+        QDragMoveEvent *dragEvent = static_cast<QDragMoveEvent*>(event);
+        if (dragEvent->mimeData()->hasFormat("application/rekonq-bookmark") || dragEvent->mimeData()->hasFormat("text/uri-list") || dragEvent->mimeData()->hasFormat("text/plain"))
+        {
+            QAction *overAction = toolBar()->actionAt(dragEvent->pos());
+            KBookmarkActionInterface *overActionBK = dynamic_cast<KBookmarkActionInterface*>(overAction);
+            QWidget *widgetAction = toolBar()->widgetForAction(overAction);
+
+            if (overAction != m_dropAction && overActionBK && widgetAction && m_dropAction)
+            {
+                toolBar()->removeAction(m_dropAction);
+                if (m_checkedAction)
                 {
-                    widget->removeEventFilter(this);
+                    m_checkedAction->setCheckable(false);
+                    m_checkedAction->setChecked(false);
                 }
-            }
-        }
-        else if (event->type() == QEvent::ParentChange)
-        {
-            QActionEvent *actionEvent = static_cast<QActionEvent*>(event);
-            if (actionEvent && actionEvent->action() != m_dropAction)
-            {
-                QWidget *widget = toolBar()->widgetForAction(actionEvent->action());
-                if (widget)
+
+                if (!overActionBK->bookmark().isGroup())
                 {
-                    widget->removeEventFilter(this);
-                }
-            }
-        }
-        else if (event->type() == QEvent::DragEnter)
-        {
-            QDragEnterEvent *dragEvent = static_cast<QDragEnterEvent*>(event);
-            if (dragEvent->mimeData()->hasFormat("application/rekonq-bookmark") || dragEvent->mimeData()->hasFormat("text/uri-list"))
-            {
-                QFrame* dropIndicatorWidget = new QFrame(toolBar());
-                dropIndicatorWidget->setFrameShape(QFrame::VLine);
-                m_dropAction = toolBar()->insertWidget(toolBar()->actionAt(dragEvent->pos()), dropIndicatorWidget);
-
-                dragEvent->accept();
-            }
-        }
-        else if (event->type() == QEvent::DragMove)
-        {
-            QDragMoveEvent *dragEvent = static_cast<QDragMoveEvent*>(event);
-            if (dragEvent->mimeData()->hasFormat("application/rekonq-bookmark") || dragEvent->mimeData()->hasFormat("text/uri-list"))
-            {
-                QAction *overAction = toolBar()->actionAt(dragEvent->pos());
-                KBookmarkActionInterface *overActionBK = dynamic_cast<KBookmarkActionInterface*>(overAction);
-                QWidget *widgetAction = toolBar()->widgetForAction(overAction);
-
-                if (overAction != m_dropAction && overActionBK && widgetAction && m_dropAction)
-                {
-                    toolBar()->removeAction(m_dropAction);
-
                     if ((dragEvent->pos().x() - widgetAction->pos().x()) > (widgetAction->width() / 2))
                     {
                         if (toolBar()->actions().count() >  toolBar()->actions().indexOf(overAction) + 1)
@@ -307,66 +351,105 @@ bool BookmarkToolBar::eventFilter(QObject *watched, QEvent *event)
                     {
                         toolBar()->insertAction(overAction, m_dropAction);
                     }
-
-                    dragEvent->accept();
-                }
-            }
-        }
-        else if (event->type() == QEvent::DragLeave)
-        {
-            QDragLeaveEvent *dragEvent = static_cast<QDragLeaveEvent*>(event);
-            delete m_dropAction;
-            m_dropAction = 0;
-            dragEvent->accept();
-        }
-        else if (event->type() == QEvent::Drop)
-        {
-            QDropEvent *dropEvent = static_cast<QDropEvent*>(event);
-            KBookmark bookmark;
-            KBookmarkGroup root = rApp->bookmarkProvider()->rootGroup();
-
-            if (dropEvent->mimeData()->hasFormat("application/rekonq-bookmark"))
-            {
-                QByteArray addresses = dropEvent->mimeData()->data("application/rekonq-bookmark");
-                bookmark =  rApp->bookmarkProvider()->bookmarkManager()->findByAddress(QString::fromLatin1(addresses.data()));
-                if (bookmark.isNull())
-                    return false;
-            }
-            else if (dropEvent->mimeData()->hasFormat("text/uri-list"))
-            {
-                QString title = dropEvent->mimeData()->text();
-                QString url = dropEvent->mimeData()->urls().at(0).toString();
-                bookmark = root.addBookmark(title, url);
-            }
-            else
-            {
-                return false;
-            }
-
-            QAction *destAction = toolBar()->actionAt(dropEvent->pos());
-            if (destAction && destAction == m_dropAction)
-            {
-                if (toolBar()->actions().indexOf(m_dropAction) > 0)
-                {
-                    destAction = toolBar()->actions().at(toolBar()->actions().indexOf(m_dropAction) - 1);
                 }
                 else
                 {
-                    destAction = toolBar()->actions().at(1);
+                    if ((dragEvent->pos().x() - widgetAction->pos().x()) >= (widgetAction->width() * 0.75))
+                    {
+                        if (toolBar()->actions().count() >  toolBar()->actions().indexOf(overAction) + 1)
+                        {
+                            toolBar()->insertAction(toolBar()->actions().at(toolBar()->actions().indexOf(overAction) + 1), m_dropAction);
+                        }
+                        else
+                        {
+                            toolBar()->addAction(m_dropAction);
+                        }
+                    }
+                    else if ((dragEvent->pos().x() - widgetAction->pos().x()) <= (widgetAction->width() * 0.25))
+                    {
+                        toolBar()->insertAction(overAction, m_dropAction);
+                    }
+                    else
+                    {
+                        overAction->setCheckable(true);
+                        overAction->setChecked(true);
+                        m_checkedAction = overAction;
+                    }
                 }
+
+                dragEvent->accept();
             }
+        }
+    }
+    break;
 
-            if (destAction)
+
+    case QEvent::Drop:
+    {
+        QDropEvent *dropEvent = static_cast<QDropEvent*>(event);
+        KBookmark bookmark;
+        KBookmarkGroup root = rApp->bookmarkProvider()->bookmarkManager()->toolbar();
+
+        if (dropEvent->mimeData()->hasFormat("application/rekonq-bookmark"))
+        {
+            QByteArray addresses = dropEvent->mimeData()->data("application/rekonq-bookmark");
+            bookmark =  rApp->bookmarkProvider()->bookmarkManager()->findByAddress(QString::fromLatin1(addresses.data()));
+            if (bookmark.isNull())
+                return false;
+        }
+        else if (dropEvent->mimeData()->hasFormat("text/uri-list"))
+        {
+            // DROP is URL
+            QString url = dropEvent->mimeData()->urls().at(0).toString();
+            QString title = url.contains(rApp->mainWindow()->currentTab()->url().url())
+                            ? rApp->mainWindow()->currentTab()->view()->title()
+                            : url;
+            bookmark = root.addBookmark(title, url);
+        }
+        else if (dropEvent->mimeData()->hasFormat("text/plain"))
+        {
+            // DROP is TEXT
+            QString url = dropEvent->mimeData()->text();
+            KUrl u(url);
+            if (u.isValid())
             {
-                KBookmarkActionInterface *destBookmarkAction = dynamic_cast<KBookmarkActionInterface *>(destAction);
-                QWidget *widgetAction = toolBar()->widgetForAction(destAction);
+                QString title = url.contains(rApp->mainWindow()->currentTab()->url().url())
+                                ? rApp->mainWindow()->currentTab()->view()->title()
+                                : url;
+                bookmark = root.addBookmark(title, url);
+            }
+        }
+        else
+        {
+            return false;
+        }
 
-                if (destBookmarkAction && !destBookmarkAction->bookmark().isNull() && widgetAction
-                        && bookmark.address() != destBookmarkAction->bookmark().address())
+        QAction *destAction = toolBar()->actionAt(dropEvent->pos());
+        if (destAction && destAction == m_dropAction)
+        {
+            if (toolBar()->actions().indexOf(m_dropAction) > 0)
+            {
+                destAction = toolBar()->actions().at(toolBar()->actions().indexOf(m_dropAction) - 1);
+            }
+            else
+            {
+                destAction = toolBar()->actions().at(1);
+            }
+        }
+
+        if (destAction)
+        {
+            KBookmarkActionInterface *destBookmarkAction = dynamic_cast<KBookmarkActionInterface *>(destAction);
+            QWidget *widgetAction = toolBar()->widgetForAction(destAction);
+
+            if (destBookmarkAction && !destBookmarkAction->bookmark().isNull() && widgetAction
+                    && bookmark.address() != destBookmarkAction->bookmark().address())
+            {
+                KBookmark destBookmark = destBookmarkAction->bookmark();
+
+                if (!destBookmark.isGroup())
                 {
-                    KBookmark destBookmark = destBookmarkAction->bookmark();
-
-                    if ((dropEvent->pos().x() - widgetAction->pos().x()) > (widgetAction->width() / 2))
+                    if ((dropEvent->pos().x() - widgetAction->pos().x()) >= (widgetAction->width() / 2))
                     {
                         root.moveBookmark(bookmark, destBookmark);
                     }
@@ -374,28 +457,52 @@ bool BookmarkToolBar::eventFilter(QObject *watched, QEvent *event)
                     {
                         root.moveBookmark(bookmark, destBookmark.parentGroup().previous(destBookmark));
                     }
-
-                    rApp->bookmarkProvider()->bookmarkManager()->emitChanged();
                 }
-            }
-            else
-            {
-                root.deleteBookmark(bookmark);
-                bookmark = root.addBookmark(bookmark);
-                if (dropEvent->pos().x() < toolBar()->widgetForAction(toolBar()->actions().first())->pos().x())
+                else
                 {
-                    root.moveBookmark(bookmark, KBookmark());
+                    if ((dropEvent->pos().x() - widgetAction->pos().x()) >= (widgetAction->width() * 0.75))
+                    {
+                        root.moveBookmark(bookmark, destBookmark);
+                    }
+                    else if ((dropEvent->pos().x() - widgetAction->pos().x()) <= (widgetAction->width() * 0.25))
+                    {
+                        root.moveBookmark(bookmark, destBookmark.parentGroup().previous(destBookmark));
+                    }
+                    else
+                    {
+                        destBookmark.toGroup().addBookmark(bookmark);
+                    }
                 }
 
+
                 rApp->bookmarkProvider()->bookmarkManager()->emitChanged();
             }
-            dropEvent->accept();
         }
+        else
+        {
+            root.deleteBookmark(bookmark);
+            bookmark = root.addBookmark(bookmark);
+            if (dropEvent->pos().x() < toolBar()->widgetForAction(toolBar()->actions().first())->pos().x())
+            {
+                root.moveBookmark(bookmark, KBookmark());
+            }
+
+            rApp->bookmarkProvider()->bookmarkManager()->emitChanged();
+        }
+        dropEvent->accept();
     }
-    else
+    break;
+
+    default:
+        break;
+    }
+
+    // These events need to be handled only for Bookmark actions and not the bar
+    if (watched != toolBar())
     {
-        // Drag handling
-        if (event->type() == QEvent::MouseButtonPress)
+        switch (event->type())
+        {
+        case QEvent::MouseButtonPress: // drag handling
         {
             QPoint pos = toolBar()->mapFromGlobal(QCursor::pos());
             KBookmarkActionInterface *action = dynamic_cast<KBookmarkActionInterface *>(toolBar()->actionAt(pos));
@@ -410,7 +517,9 @@ bool BookmarkToolBar::eventFilter(QObject *watched, QEvent *event)
                     return true;
             }
         }
-        else if (event->type() == QEvent::MouseMove)
+        break;
+
+        case QEvent::MouseMove:
         {
             int distance = (toolBar()->mapFromGlobal(QCursor::pos()) - m_startDragPos).manhattanLength();
             if (!m_currentMenu && distance >= QApplication::startDragDistance())
@@ -418,7 +527,9 @@ bool BookmarkToolBar::eventFilter(QObject *watched, QEvent *event)
                 startDrag();
             }
         }
-        else if (event->type() == QEvent::MouseButtonRelease)
+        break;
+
+        case QEvent::MouseButtonRelease:
         {
             int distance = (toolBar()->mapFromGlobal(QCursor::pos()) - m_startDragPos).manhattanLength();
             KBookmarkActionInterface *action = dynamic_cast<KBookmarkActionInterface *>(toolBar()->actionAt(m_startDragPos));
@@ -430,6 +541,11 @@ bool BookmarkToolBar::eventFilter(QObject *watched, QEvent *event)
                 menu->menu()->popup(QPoint(actionPos.x(), actionPos.y() + toolBar()->widgetForAction(menu)->height()));
             }
         }
+        break;
+
+        default:
+            break;
+        }
     }
 
     return QObject::eventFilter(watched, event);
