@@ -60,11 +60,11 @@
 QString temporaryUglyHackString = "";
 
 MainView::MainView(MainWindow *parent)
-        : KTabWidget(parent)
-        , m_widgetBar(new StackedUrlBar(this))
-        , m_addTabButton(0)
-        , m_currentTabIndex(0)
-        , m_parentWindow(parent)
+    : KTabWidget(parent)
+    , m_widgetBar(new StackedUrlBar(this))
+    , m_addTabButton(0)
+    , m_currentTabIndex(0)
+    , m_parentWindow(parent)
 {
     // setting tabbar
     TabBar *tabBar = new TabBar(this);
@@ -93,6 +93,7 @@ MainView::MainView(MainWindow *parent)
 
     // current page index changing
     connect(this, SIGNAL(currentChanged(int)), this, SLOT(currentChanged(int)));
+    connect(this, SIGNAL(currentChanged(int)), rApp->sessionManager(), SLOT(saveSession()));
 
     QTimer::singleShot(0, this, SLOT(postLaunch()));
 }
@@ -100,21 +101,15 @@ MainView::MainView(MainWindow *parent)
 
 void MainView::postLaunch()
 {
-    QStringList list = rApp->sessionManager()->closedSites();
-    Q_FOREACH(const QString &line, list)
+    QList<TabHistory> list = rApp->sessionManager()->closedSites();
+    Q_FOREACH(const TabHistory & tab, list)
     {
-        if (line.startsWith(QL1S("about")))
-            break;
-        QString title = line;
-        QString url = title;
-        HistoryItem item(url, QDateTime(), title);
-        m_recentlyClosedTabs.removeAll(item);
-        m_recentlyClosedTabs.prepend(item);
+        if (tab.url.startsWith(QL1S("about")))
+            continue;
+        m_recentlyClosedTabs.removeAll(tab);
+        m_recentlyClosedTabs.prepend(tab);
     }
 
-    // Session Manager
-    connect(this, SIGNAL(tabsChanged()), rApp->sessionManager(), SLOT(saveSession()));
-
     m_addTabButton->setDefaultAction(m_parentWindow->actionByName("new_tab"));
 
     m_addTabButton->setAutoRaise(true);
@@ -139,7 +134,7 @@ WebTab *MainView::currentWebTab() const
 {
     int i = currentIndex();
     WebTab *b = webTab(i);
-    if(b)
+    if (b)
         return b;
 
     kDebug() << "We failed to find the current tab!!! Let's go sure with the first one...";
@@ -204,23 +199,14 @@ void MainView::updateTabBar()
 
 void MainView::webReload()
 {
-    WebTab *webTab = currentWebTab();
-    if (webTab->view()->url().scheme() != QL1S("about"))
-    {
-        QAction *action = webTab->view()->page()->action(QWebPage::Reload);
-        action->trigger();
-    }
-    else
-    {
-        webTab->view()->setUrl(webTab->page()->loadingUrl());
-    }
+    reloadTab(currentIndex());
 }
 
 
 void MainView::webStop()
 {
-    WebTab *webTab = currentWebTab();
-    QAction *action = webTab->view()->page()->action(QWebPage::Stop);
+    WebTab *tabToStop = currentWebTab();
+    QAction *action = tabToStop->view()->page()->action(QWebPage::Stop);
     action->trigger();
 }
 
@@ -233,7 +219,16 @@ void MainView::reloadTab(int index)
     if (index < 0 || index >= count())
         return;
 
-    webTab(index)->view()->reload();
+    WebTab *reloadingTab = webTab(index);
+    if (reloadingTab->view()->url().scheme() != QL1S("about"))
+    {
+        QAction *action = reloadingTab->view()->page()->action(QWebPage::Reload);
+        action->trigger();
+    }
+    else
+    {
+        reloadingTab->view()->setUrl(reloadingTab->page()->loadingUrl());
+    }
 }
 
 
@@ -244,10 +239,10 @@ void MainView::currentChanged(int index)
     WebTab *tab = this->webTab(index);
     if (!tab)
         return;
-    
+
     // retrieve the old webview (that where we move from)
     WebTab *oldTab = this->webTab(m_currentTabIndex);
-    
+
     // set current index
     m_currentTabIndex = index;
 
@@ -281,8 +276,6 @@ void MainView::currentChanged(int index)
         tab->view()->setFocus();
 
     tabBar()->resetTabHighlighted(index);
-
-    emit tabsChanged();
 }
 
 
@@ -334,12 +327,6 @@ WebTab *MainView::newWebTab(bool focused)
     {
         setCurrentWidget(tab);
     }
-    else
-    {
-        // if tab is not focused,
-        // current index doesn't change...
-        emit tabsChanged();
-    }
 
     return tab;
 }
@@ -349,6 +336,8 @@ void MainView::newTab()
 {
     WebView *w = newWebTab()->view();
 
+    currentUrlBar()->setFocus();
+
     switch (ReKonfig::newTabsBehaviour())
     {
     case 0: // new tab page
@@ -363,7 +352,6 @@ void MainView::newTab()
     default:
         break;
     }
-    currentUrlBar()->setFocus();
 }
 
 
@@ -371,7 +359,7 @@ void MainView::reloadAllTabs()
 {
     for (int i = 0; i < count(); ++i)
     {
-        webTab(i)->view()->reload();
+        reloadTab(i);
     }
 }
 
@@ -394,7 +382,6 @@ void MainView::windowCloseRequested()
         }
         return;
     }
-    kDebug() << "Invalid tab index" << "line:" << __LINE__;
 }
 
 
@@ -437,9 +424,21 @@ void MainView::cloneTab(int index)
 // When index is -1 index chooses the current tab
 void MainView::closeTab(int index, bool del)
 {
-    // open default homePage if just one tab is opened
+    if (index < 0)
+        index = currentIndex();
+    if (index < 0 || index >= count())
+        return;
+
     if (count() == 1)
     {
+        if (ReKonfig::lastTabClosesWindow())
+        {
+            // closing window...
+            m_parentWindow->close();
+            return;
+        }
+
+        // open default homePage if just one tab is opened
         WebView *w = currentWebTab()->view();
 
         if (currentWebTab()->url().protocol() == QL1S("about"))
@@ -461,11 +460,6 @@ void MainView::closeTab(int index, bool del)
         return;
     }
 
-    if (index < 0)
-        index = currentIndex();
-    if (index < 0 || index >= count())
-        return;
-
     WebTab *tabToClose = webTab(index);
     if (!tabToClose)
         return;
@@ -486,13 +480,12 @@ void MainView::closeTab(int index, bool del)
        )
     {
         const int recentlyClosedTabsLimit = 10;
-        QString title = tabToClose->view()->title();
-        QString url = tabToClose->url().prettyUrl();
-        HistoryItem item(url, QDateTime(), title);
-        m_recentlyClosedTabs.removeAll(item);
+        TabHistory history(tabToClose->view()->history());
+
+        m_recentlyClosedTabs.removeAll(history);
         if (m_recentlyClosedTabs.count() == recentlyClosedTabsLimit)
             m_recentlyClosedTabs.removeLast();
-        m_recentlyClosedTabs.prepend(item);
+        m_recentlyClosedTabs.prepend(history);
     }
 
     removeTab(index);
@@ -505,13 +498,8 @@ void MainView::closeTab(int index, bool del)
     {
         tabToClose->deleteLater();
     }
-
-    // if tab was not focused, current index does not change...
-    if (index != currentIndex())
-        emit tabsChanged();
 }
 
-
 void MainView::webViewLoadStarted()
 {
     WebView *view = qobject_cast<WebView *>(sender());
@@ -530,6 +518,11 @@ void MainView::webViewLoadStarted()
 
     emit browserTabLoading(true);
     emit showStatusBarMessage(i18n("Loading..."), Rekonq::Info);
+
+    if (view == currentWebTab()->view() && view->url().scheme() != QL1S("about"))
+    {
+        view->setFocus();
+    }
 }
 
 
@@ -619,8 +612,6 @@ void MainView::webViewUrlChanged(const QUrl &url)
     }
     if (ReKonfig::hoveringTabOption() == 2)
         tabBar()->setTabToolTip(index, webTab(index)->url().toMimeDataString());
-
-    emit tabsChanged();
 }
 
 
@@ -647,8 +638,10 @@ void MainView::openLastClosedTab()
     if (m_recentlyClosedTabs.isEmpty())
         return;
 
-    const HistoryItem item = m_recentlyClosedTabs.takeFirst();
-    rApp->loadUrl(KUrl(item.url), Rekonq::NewTab);
+    TabHistory history = m_recentlyClosedTabs.takeFirst();
+    WebView *view = rApp->mainWindow()->mainView()->newWebTab()->view();
+
+    history.applyHistory(view->history());
 }
 
 
@@ -657,25 +650,38 @@ void MainView::openClosedTab()
     KAction *action = qobject_cast<KAction *>(sender());
     if (action)
     {
-        rApp->loadUrl(action->data().toUrl(), Rekonq::NewTab);
+        WebView *view = rApp->mainWindow()->mainView()->newWebTab()->view();
+        TabHistory history;
+        Q_FOREACH(TabHistory item, m_recentlyClosedTabs)
+        {
+            if (item.history == action->data().toByteArray())
+            {
+                history = item;
+                break;
+            }
+        }
+        history.applyHistory(view->history());
 
-        QString title = action->text();
-        title = title.remove('&');
-        HistoryItem item(action->data().toString(), QDateTime(), title);
-        m_recentlyClosedTabs.removeAll(item);
+        m_recentlyClosedTabs.removeAll(history);
     }
 }
 
 
-void MainView::switchToTab()
+void MainView::switchToTab(const int index)
 {
-    // uses the sender to determine the tab index
-    QAction *sender = static_cast<QAction*>(QObject::sender());
-    int index = sender->data().toInt();
-    index -= 1; // to compensate for off by 1 presented to the user
-    if (index < 0 || index >= count())
+    if (index <= 0 || index > count())
         return;
-    setCurrentIndex(index);
+    setCurrentIndex(index - 1);
+}
+
+
+void MainView::loadFavorite(const int index)
+{
+    QStringList urls = ReKonfig::previewUrls();
+    if (index < 0 || index > urls.length())
+        return;
+    KUrl url = KUrl(urls.at(index - 1));
+    rApp->loadUrl(url);
 }
 
 
@@ -718,7 +724,6 @@ void MainView::detachTab(int index, MainWindow *toWindow)
 
     WebTab *tab = webTab(index);
     KUrl u = tab->url();
-    kDebug() << "detaching tab with url: " << u;
     if (u.scheme() == QL1S("about"))
     {
         closeTab(index);
