@@ -100,19 +100,12 @@ public:
      */
     WebTab *newWebTab(bool focused = true);
 
-    inline QList<HistoryItem> recentlyClosedTabs()
+    inline QList<TabHistory> recentlyClosedTabs()
     {
         return m_recentlyClosedTabs;
     }
 
 Q_SIGNALS:
-    // tabs change when:
-    // - current tab change
-    // - one tab is closed
-    // - one tab is added
-    // - one tab is updated (eg: changes url)
-    void tabsChanged();
-
     // current tab signals
     void currentTitle(const QString &url);
     void showStatusBarMessage(const QString &message, Rekonq::Notify status = Rekonq::Info);
@@ -151,7 +144,8 @@ public Q_SLOTS:
 
     void openLastClosedTab();
     void openClosedTab();
-    void switchToTab();
+    void switchToTab(const int index);
+    void loadFavorite(const int index);
 
     // WEB slot actions
     void webReload();
@@ -198,7 +192,7 @@ private:
 
     int m_currentTabIndex;
 
-    QList<HistoryItem> m_recentlyClosedTabs;
+    QList<TabHistory> m_recentlyClosedTabs;
 
     MainWindow *m_parentWindow;
 };
