@@ -35,15 +35,18 @@
 #include "rekonq.h"
 
 // Local Includes
+#include "adblockmanager.h"
 #include "analyzerpanel.h"
 #include "application.h"
 #include "bookmarkprovider.h"
 #include "bookmarkspanel.h"
 #include "bookmarkstoolbar.h"
+#include "downloadmanager.h"
 #include "findbar.h"
 #include "historypanel.h"
 #include "iconmanager.h"
 #include "mainview.h"
+#include "sessionmanager.h"
 #include "settingsdialog.h"
 #include "stackedurlbar.h"
 #include "tabbar.h"
@@ -68,15 +71,17 @@
 #include <KMenu>
 #include <KMenuBar>
 #include <KMessageBox>
-#include <KPassivePopup>
 #include <KProcess>
 #include <KPushButton>
-#include <KStandardDirs>
-#include <KToggleFullScreenAction>
 #include <KProtocolManager>
+#include <KStandardDirs>
+#include <KTemporaryFile>
+#include <KToggleFullScreenAction>
+#include <KXMLGUIFactory>
 
 #include <KParts/Part>
 #include <KParts/BrowserExtension>
+#include <KMimeTypeTrader>
 
 // Qt Includes
 #include <QtCore/QTimer>
@@ -93,24 +98,25 @@
 
 #include <QtWebKit/QWebHistory>
 
+#include <QSignalMapper>
+
 
 MainWindow::MainWindow()
-        : KXmlGuiWindow()
-        , m_view(new MainView(this))
-        , m_findBar(new FindBar(this))
-        , m_zoomBar(new ZoomBar(this))
-        , m_historyPanel(0)
-        , m_bookmarksPanel(0)
-        , m_webInspectorPanel(0)
-        , m_analyzerPanel(0)
-        , m_historyBackMenu(0)
-        , m_historyForwardMenu(0)
-        , m_userAgentMenu(new KMenu(this))
-        , m_bookmarksBar(0)
-        , m_popup(new KPassivePopup(this))
-        , m_hidePopupTimer(new QTimer(this))
-        , m_toolsMenu(0)
-        , m_developerMenu(0)
+    : KXmlGuiWindow()
+    , m_view(new MainView(this))
+    , m_findBar(new FindBar(this))
+    , m_zoomBar(new ZoomBar(this))
+    , m_historyPanel(0)
+    , m_bookmarksPanel(0)
+    , m_webInspectorPanel(0)
+    , m_analyzerPanel(0)
+    , m_historyBackMenu(0)
+    , m_historyForwardMenu(0)
+    , m_userAgentMenu(new KMenu(this))
+    , m_bookmarksBar(0)
+    , m_popup(new QLabel(this))
+    , m_hidePopupTimer(new QTimer(this))
+    , m_rekonqMenu(0)
 {
     // creating a centralWidget containing panel, m_view and the hidden findbar
     QWidget *centralWidget = new QWidget;
@@ -142,9 +148,6 @@ MainWindow::MainWindow()
     // setting up rekonq toolbar(s)
     setupToolbars();
 
-    // disable help menu, as we'll load it on our own...
-    setHelpMenuEnabled(false);
-        
     // a call to KXmlGuiWindow::setupGUI() populates the GUI
     // with actions, using KXMLGUI.
     // It also applies the saved mainwindow settings, if any, and ask the
@@ -166,7 +169,7 @@ MainWindow::MainWindow()
 MainWindow::~MainWindow()
 {
     m_hidePopupTimer->stop();
-    
+
     rApp->bookmarkProvider()->removeBookmarkBar(m_bookmarksBar);
     rApp->bookmarkProvider()->removeBookmarkPanel(m_bookmarksPanel);
     rApp->removeMainWindow(this);
@@ -201,6 +204,16 @@ void MainWindow::initBookmarkBar()
     }
     m_bookmarksBar = new BookmarkToolBar(XMLGUIBkBar, this);
     rApp->bookmarkProvider()->registerBookmarkBar(m_bookmarksBar);
+    QAction *a = actionByName(QL1S("show_bookmarks_toolbar"));
+    a->setChecked(XMLGUIBkBar->isVisible());
+    connect(a, SIGNAL(toggled(bool)), this, SLOT(toggleBookmarkBarVisible(bool)));
+}
+
+
+void MainWindow::toggleBookmarkBarVisible(bool visible)
+{
+    if (m_bookmarksBar)
+        m_bookmarksBar->toolBar()->setVisible(visible);
 }
 
 
@@ -216,76 +229,22 @@ void MainWindow::configureToolbars()
 }
 
 
-void MainWindow::updateToolsMenu()
-{
-    if (m_toolsMenu->isEmpty())
-    {
-        m_toolsMenu->addAction(actionByName(KStandardAction::name(KStandardAction::Open)));
-        m_toolsMenu->addAction(actionByName(KStandardAction::name(KStandardAction::SaveAs)));
-        m_toolsMenu->addAction(actionByName(KStandardAction::name(KStandardAction::Print)));
-        m_toolsMenu->addAction(actionByName(KStandardAction::name(KStandardAction::Find)));
-
-        QAction *action = actionByName(KStandardAction::name(KStandardAction::Zoom));
-        action->setCheckable(true);
-        connect(m_zoomBar, SIGNAL(visibilityChanged(bool)), action, SLOT(setChecked(bool)));
-        m_toolsMenu->addAction(action);
-
-        m_toolsMenu->addAction(actionByName(QL1S("useragent")));
-
-        m_toolsMenu->addSeparator();
-
-        m_toolsMenu->addAction(actionByName(QL1S("private_browsing")));
-        m_toolsMenu->addAction(actionByName(QL1S("clear_private_data")));
-
-        m_toolsMenu->addSeparator();
-
-        m_developerMenu = new KActionMenu(KIcon("applications-development-web"), i18n("Development"), this);
-        m_developerMenu->addAction(actionByName(QL1S("web_inspector")));
-        m_developerMenu->addAction(actionByName(QL1S("page_source")));
-        m_developerMenu->addAction(actionByName(QL1S("net_analyzer")));
-
-        m_toolsMenu->addAction(m_developerMenu);
-        if (!ReKonfig::showDeveloperTools())
-            m_developerMenu->setVisible(false);
-
-        m_toolsMenu->addSeparator();
-
-        action = m_bookmarksBar->toolBar()->toggleViewAction();
-        action->setText(i18n("Bookmarks Toolbar"));
-        action->setIcon(KIcon("bookmarks-bar"));
-        m_toolsMenu->addAction(action);
-
-        m_toolsMenu->addAction(actionByName(QL1S("show_history_panel")));
-        m_toolsMenu->addAction(actionByName(QL1S("show_bookmarks_panel")));
-        m_toolsMenu->addAction(actionByName(KStandardAction::name(KStandardAction::FullScreen)));
-
-        m_toolsMenu->addSeparator();
-
-        helpMenu()->setIcon(KIcon("help-browser"));
-        m_toolsMenu->addAction(helpMenu()->menuAction());
-        m_toolsMenu->addAction(actionByName(KStandardAction::name(KStandardAction::Preferences)));
-    }
-
-    m_developerMenu->setVisible(ReKonfig::showDeveloperTools());
-}
-
-
 void MainWindow::postLaunch()
 {
     setupBookmarksAndToolsShortcuts();
 
     // setting popup notification
-    m_popup->setAutoDelete(false);
     connect(rApp, SIGNAL(focusChanged(QWidget*, QWidget*)), m_popup, SLOT(hide()));
-    m_popup->setFrameShape(QFrame::NoFrame);
-    m_popup->setLineWidth(0);
+    m_popup->setAutoFillBackground(true);
+    m_popup->raise();
+    m_popup->hide();
     connect(m_hidePopupTimer, SIGNAL(timeout()), m_popup, SLOT(hide()));
 
     // notification system
     connect(m_view, SIGNAL(showStatusBarMessage(const QString&, Rekonq::Notify)), this, SLOT(notifyMessage(const QString&, Rekonq::Notify)));
     connect(m_view, SIGNAL(linkHovered(const QString&)), this, SLOT(notifyMessage(const QString&)));
 
-    // --------- connect signals and slots
+    // connect signals and slots
     connect(m_view, SIGNAL(currentTitle(const QString &)), this, SLOT(updateWindowTitle(const QString &)));
     connect(m_view, SIGNAL(printRequested(QWebFrame *)), this, SLOT(printRequested(QWebFrame *)));
 
@@ -300,7 +259,7 @@ void MainWindow::postLaunch()
     // update toolbar actions signals
     connect(m_view, SIGNAL(currentChanged(int)), this, SLOT(updateActions()));
 
-    //Change window icon according to tab icon
+    // Change window icon according to tab icon
     connect(m_view, SIGNAL(currentChanged(int)), this, SLOT(changeWindowIcon(int)));
 
     // launch it manually. Just the first time...
@@ -311,9 +270,13 @@ void MainWindow::postLaunch()
 
     // Zoom Bar signal
     connect(m_view, SIGNAL(currentChanged(int)), m_zoomBar, SLOT(updateSlider(int)));
+
     // Ctrl + wheel handling
     connect(this->currentTab()->view(), SIGNAL(zoomChanged(int)), m_zoomBar, SLOT(setValue(int)));
 
+    // Save session on window closing
+    connect(this, SIGNAL(windowClosing()), rApp->sessionManager(), SLOT(saveSession()));
+
     // setting up toolbars to NOT have context menu enabled
     setContextMenuPolicy(Qt::DefaultContextMenu);
 
@@ -332,6 +295,7 @@ QSize MainWindow::sizeHint() const
     return size;
 }
 
+
 void MainWindow::changeWindowIcon(int index)
 {
     if (ReKonfig::useFavicon())
@@ -342,6 +306,7 @@ void MainWindow::changeWindowIcon(int index)
     }
 }
 
+
 void MainWindow::setupActions()
 {
     // this let shortcuts work..
@@ -359,7 +324,7 @@ void MainWindow::setupActions()
     KStandardAction::open(this, SLOT(fileOpen()), actionCollection());
     KStandardAction::saveAs(this, SLOT(fileSaveAs()), actionCollection());
     KStandardAction::print(this, SLOT(printRequested()), actionCollection());
-    KStandardAction::quit(this , SLOT(close()), actionCollection());
+    KStandardAction::quit(rApp, SLOT(queryQuit()), actionCollection());
 
     a = KStandardAction::find(m_findBar, SLOT(show()), actionCollection());
     KShortcut findShortcut = KStandardShortcut::find();
@@ -434,7 +399,8 @@ void MainWindow::setupActions()
 
     // ========================= History related actions ==============================
     a = actionCollection()->addAction(KStandardAction::Back);
-    connect(a, SIGNAL(triggered(Qt::MouseButtons, Qt::KeyboardModifiers)), this, SLOT(openPrevious(Qt::MouseButtons, Qt::KeyboardModifiers)));
+    connect(a, SIGNAL(triggered(Qt::MouseButtons, Qt::KeyboardModifiers)),
+            this, SLOT(openPrevious(Qt::MouseButtons, Qt::KeyboardModifiers)));
 
     m_historyBackMenu = new KMenu(this);
     a->setMenu(m_historyBackMenu);
@@ -442,7 +408,8 @@ void MainWindow::setupActions()
     connect(m_historyBackMenu, SIGNAL(triggered(QAction *)), this, SLOT(openActionUrl(QAction *)));
 
     a = actionCollection()->addAction(KStandardAction::Forward);
-    connect(a, SIGNAL(triggered(Qt::MouseButtons, Qt::KeyboardModifiers)), this, SLOT(openNext(Qt::MouseButtons, Qt::KeyboardModifiers)));
+    connect(a, SIGNAL(triggered(Qt::MouseButtons, Qt::KeyboardModifiers)),
+            this, SLOT(openNext(Qt::MouseButtons, Qt::KeyboardModifiers)));
 
     m_historyForwardMenu = new KMenu(this);
     a->setMenu(m_historyForwardMenu);
@@ -479,15 +446,29 @@ void MainWindow::setupActions()
     closedTabsMenu->setDelayed(false);
     actionCollection()->addAction(QL1S("closed_tab_menu"), closedTabsMenu);
 
+    QSignalMapper *tabSignalMapper = new QSignalMapper(this);
     // shortcuts for quickly switching to a tab
     for (int i = 1; i <= 9; i++)
     {
         a = new KAction(i18n("Switch to Tab %1", i), this);
         a->setShortcut(KShortcut(QString("Alt+%1").arg(i)));
-        a->setData(QVariant(i));
-        actionCollection()->addAction(QL1S(("switch_tab_" + QString::number(i)).toAscii()), a);
-        connect(a, SIGNAL(triggered(bool)), m_view, SLOT(switchToTab()));
+        actionCollection()->addAction(QL1S(QString("switch_tab_" + QString::number(i)).toAscii()), a);
+        connect(a, SIGNAL(triggered(bool)), tabSignalMapper, SLOT(map()));
+        tabSignalMapper->setMapping(a, i);
     }
+    connect(tabSignalMapper, SIGNAL(mapped(const int)), m_view, SLOT(switchToTab(const int)));
+
+    // shortcuts for loading favorite pages
+    QSignalMapper *favoritesSignalMapper = new QSignalMapper(this);
+    for (int i = 1; i <= 9; ++i)
+    {
+        a = new KAction(i18n("Switch to Favorite Page %1", i), this);
+        a->setShortcut(KShortcut(QString("Ctrl+%1").arg(i)));
+        actionCollection()->addAction(QL1S(QString("switch_favorite_" + QString::number(i)).toAscii()), a);
+        connect(a, SIGNAL(triggered(bool)), favoritesSignalMapper, SLOT(map()));
+        favoritesSignalMapper->setMapping(a, i);
+    }
+    connect(favoritesSignalMapper, SIGNAL(mapped(const int)), m_view, SLOT(loadFavorite(const int)));
 
     // ============================== Indexed Tab Actions ====================================
     a = new KAction(KIcon("tab-close"), i18n("&Close Tab"), this);
@@ -519,7 +500,12 @@ void MainWindow::setupActions()
     bmMenu->setShortcut(KShortcut(Qt::ALT + Qt::Key_B));
     actionCollection()->addAction(QL1S("bookmarksActionMenu"), bmMenu);
 
-    // --- User Agent
+    // Bookmark Toolbar
+    a = new KAction(KIcon("bookmarks-bar"), i18n("Bookmarks Toolbar"), this);
+    a->setCheckable(true);
+    actionCollection()->addAction(QL1S("show_bookmarks_toolbar"), a);
+
+    // User Agent
     a = new KAction(KIcon("preferences-web-browser-identification"), i18n("Browser Identification"), this);
     actionCollection()->addAction(QL1S("useragent"), a);
     a->setMenu(m_userAgentMenu);
@@ -528,6 +514,17 @@ void MainWindow::setupActions()
     a = new KAction(KIcon("preferences-web-browser-identification"), i18n("Browser Identification"), this);
     actionCollection()->addAction(QL1S("UserAgentSettings"), a);
     connect(a, SIGNAL(triggered(bool)), this, SLOT(showUserAgentSettings()));
+
+    // Editable Page
+    a = new KAction(KIcon("document-edit"), i18n("Set Editable"), this);
+    a->setCheckable(true);
+    actionCollection()->addAction(QL1S("set_editable"), a);
+    connect(a, SIGNAL(triggered(bool)), this, SLOT(setEditable(bool)));
+
+    // Adblock
+    a = new KAction(KIcon("preferences-web-browser-adblock"), i18n("Ad Block"), this);
+    actionCollection()->addAction(QL1S("adblock"), a);
+    connect(a, SIGNAL(triggered(bool)), rApp->adblockManager(), SLOT(showSettings()));
 }
 
 
@@ -537,14 +534,11 @@ void MainWindow::setupTools()
     toolsAction->setDelayed(false);
     toolsAction->setShortcutConfigurable(true);
     toolsAction->setShortcut(KShortcut(Qt::ALT + Qt::Key_T));
-    m_toolsMenu = new KMenu(this);
-    toolsAction->setMenu(m_toolsMenu); // dummy menu to have the dropdown arrow
-    connect(m_toolsMenu, SIGNAL(aboutToShow()), this, SLOT(updateToolsMenu()));
+    m_rekonqMenu = new KMenu(this);
+    toolsAction->setMenu(m_rekonqMenu); // dummy menu to have the dropdown arrow
 
     // adding rekonq_tools to rekonq actionCollection
     actionCollection()->addAction(QL1S("rekonq_tools"), toolsAction);
-
-    // Actions are added after the call to setupGUI() to ensure the help menu works
 }
 
 
@@ -555,7 +549,8 @@ void MainWindow::setupPanels()
     // STEP 1
     // Setup history panel
     m_historyPanel = new HistoryPanel(i18n("History Panel"), this);
-    connect(m_historyPanel, SIGNAL(openUrl(const KUrl&, const Rekonq::OpenType &)), rApp, SLOT(loadUrl(const KUrl&, const Rekonq::OpenType &)));
+    connect(m_historyPanel, SIGNAL(openUrl(const KUrl&, const Rekonq::OpenType &)),
+            rApp, SLOT(loadUrl(const KUrl&, const Rekonq::OpenType &)));
     connect(m_historyPanel, SIGNAL(itemHovered(QString)), this, SLOT(notifyMessage(QString)));
     connect(m_historyPanel, SIGNAL(destroyed()), rApp, SLOT(saveConfiguration()));
 
@@ -570,7 +565,8 @@ void MainWindow::setupPanels()
     // STEP 2
     // Setup bookmarks panel
     m_bookmarksPanel = new BookmarksPanel(i18n("Bookmarks Panel"), this);
-    connect(m_bookmarksPanel, SIGNAL(openUrl(const KUrl&, const Rekonq::OpenType &)), rApp, SLOT(loadUrl(const KUrl&, const Rekonq::OpenType &)));
+    connect(m_bookmarksPanel, SIGNAL(openUrl(const KUrl&, const Rekonq::OpenType &)),
+            rApp, SLOT(loadUrl(const KUrl&, const Rekonq::OpenType &)));
     connect(m_bookmarksPanel, SIGNAL(itemHovered(QString)), this, SLOT(notifyMessage(QString)));
     connect(m_bookmarksPanel, SIGNAL(destroyed()), rApp, SLOT(saveConfiguration()));
 
@@ -611,6 +607,16 @@ void MainWindow::setupPanels()
     m_analyzerPanel->hide();
 }
 
+
+void MainWindow::finalizeGUI(KXMLGUIClient* client)
+{
+    KXmlGuiWindow::finalizeGUI(client);
+    //update rekonqMenu when GUI has changed
+    KMenu *m = qobject_cast<KMenu*>(factory()->container("rekonqMenu", this));
+    m_rekonqMenu->addActions(m->actions());
+}
+
+
 void MainWindow::openLocation()
 {
     if (isFullScreen())
@@ -646,6 +652,19 @@ void MainWindow::fileSaveAs()
     if (destUrl.isEmpty())
         return;
 
+    if (w->page()->isContentEditable())
+    {
+        QString code = w->page()->mainFrame()->toHtml();
+        QFile file(destUrl);
+        if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
+            return;
+
+        QTextStream out(&file);
+        out << code;
+
+        return;
+    }
+
     KIO::Job *job = KIO::file_copy(srcUrl, KUrl(destUrl), -1, KIO::Overwrite);
     job->addMetaData("MaxCacheSize", "0");  // Don't store in http cache.
     job->addMetaData("cache", "cache");     // Use entry from cache if available.
@@ -673,7 +692,6 @@ void MainWindow::preferences()
 
 void MainWindow::updateActions()
 {
-    kDebug() << "updating actions..";
     bool rekonqPage = currentTab()->page()->isOnRekonqPage();
 
     QAction *historyBackAction = actionByName(KStandardAction::name(KStandardAction::Back));
@@ -938,11 +956,36 @@ QString MainWindow::selectedText() const
 
 void MainWindow::viewPageSource()
 {
-    if (!currentTab())
+    WebTab * w = currentTab();
+
+    if (!w)
         return;
 
-    KUrl url = currentTab()->url();
-    KRun::runUrl(url, QL1S("text/plain"), this, false);
+    KUrl url = w->url();
+    QString code = w->page()->mainFrame()->toHtml();
+
+    KTemporaryFile tmpFile;
+    tmpFile.setAutoRemove(false);
+    if (!tmpFile.open())
+        return;
+
+    QTextStream out(&tmpFile);
+    out << code;
+    tmpFile.close();
+    KUrl tmpUrl(tmpFile.fileName());
+
+    KParts::ReadOnlyPart *pa = KMimeTypeTrader::createPartInstanceFromQuery<KParts::ReadOnlyPart>(QL1S("text/plain"), w, this, QString());
+    if (pa)
+    {
+        WebTab *srcTab = m_view->newWebTab(true);
+        srcTab->page()->setIsOnRekonqPage(true);
+        srcTab->setPart(pa, tmpUrl);
+        srcTab->urlBar()->setQUrl(url.pathOrUrl());
+        m_view->setTabText(m_view->currentIndex(), i18n("Source of: ") + url.prettyUrl());
+        updateActions();
+    }
+    else
+        KRun::runUrl(tmpUrl, QL1S("text/plain"), this, false);
 }
 
 
@@ -1111,11 +1154,6 @@ bool MainWindow::event(QEvent *event)
 
 void MainWindow::notifyMessage(const QString &msg, Rekonq::Notify status)
 {
-    if (this != QApplication::activeWindow())
-    {
-        return;
-    }
-
     // deleting popus if empty msgs
     if (msg.isEmpty())
     {
@@ -1125,73 +1163,57 @@ void MainWindow::notifyMessage(const QString &msg, Rekonq::Notify status)
 
     m_hidePopupTimer->stop();
 
-
     switch (status)
     {
-    case Rekonq::Url:
-        break;
     case Rekonq::Info:
         m_hidePopupTimer->start(500);
         break;
+    case Rekonq::Url:
     case Rekonq::Success:
-        break;
     case Rekonq::Error:
-        break;
     case Rekonq::Download:
-        break;
     default:
         break;
     }
 
-    int margin = 4;
-
-    // setting the popup
-    QLabel *label = new QLabel(msg);
-    m_popup->setView(label);
-    QSize labelSize(label->fontMetrics().width(msg) + 2*margin, label->fontMetrics().height() + 2*margin);
-    if (labelSize.width() > width())
-    {
-        labelSize.setWidth(width());
-        label->setText(label->fontMetrics().elidedText(msg, Qt::ElideMiddle, width()));
-    }
-    m_popup->setFixedSize(labelSize);
-    m_popup->layout()->setAlignment(Qt::AlignTop);
-    m_popup->layout()->setMargin(margin);
-
     // useful values
     WebTab *tab = m_view->currentWebTab();
 
     // fix crash on window close
-    if (!tab)
+    if (!tab || !tab->page())
         return;
 
-    // fix crash on window close
-    if (!tab->page())
-        return;
+    const int margin = 4;
+    const int halfWidth = width() / 2;
 
-    bool scrollbarIsVisible = tab->page()->currentFrame()->scrollBarMaximum(Qt::Horizontal);
-    int scrollbarSize = 0;
-    if (scrollbarIsVisible)
-    {
-        //TODO: detect QStyle size
-        scrollbarSize = 17;
-    }
+    // Set Popup size
+    QFontMetrics fm = m_popup->fontMetrics();
+    QSize labelSize(fm.width(msg) + 2 * margin, fm.height() + 2 * margin);
 
-    QPoint webViewOrigin = tab->view()->mapToGlobal(QPoint(0, 0));
-    int bottomLeftY = webViewOrigin.y() + tab->page()->viewportSize().height() - labelSize.height() - scrollbarSize;
+    if (labelSize.width() > halfWidth)
+        labelSize.setWidth(halfWidth);
 
-    // setting popup in bottom-left position
-    int x = mapToGlobal(QPoint(0, 0)).x();
-    int y = bottomLeftY;
+    m_popup->setFixedSize(labelSize);
+    m_popup->setText(fm.elidedText(msg, Qt::ElideMiddle, labelSize.width()));
 
-    QPoint mousePos = tab->mapToGlobal(tab->view()->mousePos());
-    if (QRect(webViewOrigin.x() , bottomLeftY , labelSize.width() , labelSize.height()).contains(mousePos))
-    {
-        // setting popup above the mouse
-        y = bottomLeftY - labelSize.height();
-    }
+    const bool horizontalScrollbarIsVisible = tab->page()->currentFrame()->scrollBarMaximum(Qt::Horizontal);
+    const bool verticalScrollbarIsVisible = tab->page()->currentFrame()->scrollBarMaximum(Qt::Vertical);
+    const bool actionBarsVisible = m_findBar->isVisible() || m_zoomBar->isVisible();
 
-    m_popup->show(QPoint(x, y));
+    const int scrollbarExtent = style()->pixelMetric(QStyle::PM_ScrollBarExtent);
+    const int hScrollbarSize = horizontalScrollbarIsVisible ? scrollbarExtent : 0;
+    const int vScrollbarSize = verticalScrollbarIsVisible ? scrollbarExtent : 0;
+
+    const QPoint mousePos = mapFromGlobal(QCursor::pos());
+    const QPoint bottomPoint = m_view->mapTo(this, m_view->geometry().bottomLeft());
+    // +1 because bottom() returns top() + height() - 1 , see QRect doku
+    int y = bottomPoint.y() + 1 - m_popup->height() - hScrollbarSize;
+    int x = QRect(QPoint(0, y), labelSize).contains(mousePos) || actionBarsVisible
+            ? width() - labelSize.width() - vScrollbarSize
+            : 0;
+
+    m_popup->move(x, y);
+    m_popup->show();
 }
 
 
@@ -1234,7 +1256,7 @@ void MainWindow::clearPrivateData()
 
         if (clearWidget.clearDownloads->isChecked())
         {
-            rApp->clearDownloadsHistory();
+            rApp->downloadManager()->clearDownloadsHistory();
         }
 
         if (clearWidget.clearCookies->isChecked())
@@ -1260,7 +1282,7 @@ void MainWindow::clearPrivateData()
             path.remove("rekonq");
             QDir cacheDir(path);
             QStringList fileList = cacheDir.entryList();
-            foreach(const QString &str, fileList)
+            foreach(const QString & str, fileList)
             {
                 QFile file(path + str);
                 file.remove();
@@ -1271,6 +1293,7 @@ void MainWindow::clearPrivateData()
     dialog->deleteLater();
 }
 
+
 void MainWindow::aboutToShowBackMenu()
 {
     m_historyBackMenu->clear();
@@ -1384,6 +1407,7 @@ void MainWindow::openActionUrl(QAction *action)
     history->goToItem(history->itemAt(index));
 }
 
+
 void MainWindow::openActionTab(QAction* action)
 {
     int index = action->data().toInt();
@@ -1406,7 +1430,6 @@ void MainWindow::setUserAgent()
 
     KUrl url = currentTab()->url();
     UserAgentInfo uaInfo;
-    kDebug() << "SETTING USER AGENT";
     uaInfo.setUserAgentForHost(uaIndex, url.host());
     currentTab()->page()->triggerAction(QWebPage::Reload);
 }
@@ -1414,7 +1437,6 @@ void MainWindow::setUserAgent()
 
 void MainWindow::populateUserAgentMenu()
 {
-    kDebug() << "populating user agent menu...";
     bool defaultUA = true;
     KUrl url = currentTab()->url();
 
@@ -1464,48 +1486,6 @@ void MainWindow::enableNetworkAnalysis(bool b)
 }
 
 
-bool MainWindow::queryClose()
-{
-    // this should fux bug 240432
-    if (rApp->sessionSaving())
-        return true;
-
-    // smooth private browsing mode
-    if (QWebSettings::globalSettings()->testAttribute(QWebSettings::PrivateBrowsingEnabled))
-        return true;
-
-    if (m_view->count() > 1)
-    {
-        int answer = KMessageBox::questionYesNoCancel(
-                         this,
-                         i18np("Are you sure you want to close the window?\n" "You have 1 tab open.",
-                               "Are you sure you want to close the window?\n" "You have %1 tabs open.",
-                               m_view->count()),
-                         i18n("Are you sure you want to close the window?"),
-                         KStandardGuiItem::quit(),
-                         KGuiItem(i18n("C&lose Current Tab"), KIcon("tab-close")),
-                         KStandardGuiItem::cancel(),
-                         "confirmClosingMultipleTabs"
-                     );
-
-        switch (answer)
-        {
-        case KMessageBox::Yes:
-            // Quit
-            return true;
-
-        case KMessageBox::No:
-            // Close only the current tab
-            m_view->closeTab();
-
-        default:
-            return false;
-        }
-    }
-    return true;
-}
-
-
 void MainWindow::saveNewToolbarConfig()
 {
     KXmlGuiWindow::saveNewToolbarConfig();
@@ -1565,3 +1545,16 @@ void MainWindow::resizeEvent(QResizeEvent *event)
 
     KMainWindow::resizeEvent(event);
 }
+
+
+void MainWindow::setEditable(bool on)
+{
+    currentTab()->page()->setContentEditable(on);
+}
+
+
+bool MainWindow::close()
+{
+    emit windowClosing();
+    return KMainWindow::close();
+}
