@@ -49,10 +49,8 @@ class WebInspectorPanel;
 class WebTab;
 class ZoomBar;
 
-class KActionMenu;
-class KPassivePopup;
-
 class QWebFrame;
+class QLabel;
 
 
 /**
@@ -117,6 +115,8 @@ public Q_SLOTS:
 
     void changeWindowIcon(int index);
 
+    bool close();
+
 Q_SIGNALS:
     // switching tabs
     void ctrlTabPressed();
@@ -125,6 +125,8 @@ Q_SIGNALS:
     void triggerPartPrint();
     void triggerPartFind();
 
+    void windowClosing();
+
 protected Q_SLOTS:
     void saveNewToolbarConfig();
 
@@ -138,7 +140,7 @@ protected:
     void moveEvent(QMoveEvent *event);
     void resizeEvent(QResizeEvent *event);
     bool event(QEvent *event);
-    bool queryClose();
+    void finalizeGUI(KXMLGUIClient *client);
 
 private Q_SLOTS:
     void postLaunch();
@@ -180,9 +182,10 @@ private Q_SLOTS:
     void showUserAgentSettings();
 
     void enableNetworkAnalysis(bool);
+    void setEditable(bool);
 
     void initBookmarkBar();
-    void updateToolsMenu();
+    void toggleBookmarkBarVisible(bool);
 
 private:
     MainView *m_view;
@@ -207,11 +210,10 @@ private:
 
     QString m_lastSearch;
 
-    KPassivePopup *m_popup;
+    QLabel *m_popup;
     QTimer *m_hidePopupTimer;
 
-    KMenu *m_toolsMenu;
-    KActionMenu *m_developerMenu;
+    KMenu *m_rekonqMenu;
 };
 
 #endif // MAINWINDOW_H
