@@ -35,6 +35,7 @@
 // Local Includes
 #include "application.h"
 #include "bookmarkprovider.h"
+#include "downloadmanager.h"
 #include "historymodels.h"
 #include "mainview.h"
 #include "mainwindow.h"
@@ -56,8 +57,8 @@
 
 
 NewTabPage::NewTabPage(QWebFrame *frame)
-        : QObject(frame)
-        , m_root(frame->documentElement())
+    : QObject(frame)
+    , m_root(frame->documentElement())
 {
     QString htmlFilePath = KStandardDirs::locate("data", "rekonq/htmls/home.html");
     QString imagesPath = QL1S("file://") + KGlobal::dirs()->findResourceDir("data", "rekonq/pics/bg.png") + QL1S("rekonq/pics");
@@ -112,7 +113,7 @@ void NewTabPage::generate(const KUrl &url)
     if (url.fileName() == QL1S("clear"))
     {
         rApp->mainWindow()->actionByName("clear_private_data")->trigger();
-        generate(QL1S("about:") + url.directory());
+        generate(QString(QL1S("about:") + url.directory()));
         return;
     }
     if (url == KUrl("about:bookmarks/edit"))
@@ -121,6 +122,9 @@ void NewTabPage::generate(const KUrl &url)
         return;
     }
 
+    // webFrame can be null. See bug:282092
+    if (!m_root.webFrame())
+        return;
     WebPage *page = qobject_cast <WebPage *>(m_root.webFrame()->page());
     page->mainFrame()->setHtml(m_html);
     page->setIsOnRekonqPage(true);
@@ -134,26 +138,31 @@ void NewTabPage::generate(const KUrl &url)
     if (encodedUrl == QByteArray("about:favorites"))
     {
         favoritesPage();
+        updateWindowIcon();
         title = i18n("Favorites");
     }
     else if (encodedUrl == QByteArray("about:closedTabs"))
     {
         closedTabsPage();
+        updateWindowIcon();
         title = i18n("Closed Tabs");
     }
     else if (encodedUrl == QByteArray("about:history"))
     {
         historyPage();
+        updateWindowIcon();
         title = i18n("History");
     }
     else if (encodedUrl == QByteArray("about:bookmarks"))
     {
         bookmarksPage();
+        updateWindowIcon();
         title = i18n("Bookmarks");
     }
     else if (encodedUrl == QByteArray("about:downloads"))
     {
         downloadsPage();
+        updateWindowIcon();
         title = i18n("Downloads");
     }
 
@@ -191,7 +200,7 @@ void NewTabPage::favoritesPage()
         else if (!WebSnap::existsImage(url))
             prev = loadingPreview(i, url);
         else
-            prev = validPreview(i, url, names.at(i));
+            prev = validPreview(i, url, QString::number(i + 1) + " - " + names.at(i));
 
         setupPreview(prev, i);
 
@@ -510,7 +519,7 @@ void NewTabPage::closedTabsPage()
 {
     m_root.addClass(QL1S("closedTabs"));
 
-    QList<HistoryItem> links = rApp->mainWindow()->mainView()->recentlyClosedTabs();
+    QList<TabHistory> links = rApp->mainWindow()->mainView()->recentlyClosedTabs();
 
     if (links.isEmpty())
     {
@@ -521,7 +530,7 @@ void NewTabPage::closedTabsPage()
 
     for (int i = 0; i < links.count(); ++i)
     {
-        HistoryItem item = links.at(i);
+        TabHistory item = links.at(i);
         QWebElement prev;
 
         if (item.url.isEmpty())
@@ -560,7 +569,7 @@ void NewTabPage::downloadsPage()
                                   KIconLoader::Toolbar);
     m_root.document().findFirst(QL1S("#actions")).appendInside(clearData);
 
-    DownloadList list = rApp->downloads();
+    DownloadList list = rApp->downloadManager()->downloads();
 
     if (list.isEmpty())
     {
@@ -569,14 +578,14 @@ void NewTabPage::downloadsPage()
         return;
     }
 
-    foreach(const DownloadItem &item, list)
+    foreach(DownloadItem * item, list)
     {
         m_root.prependInside(markup(QL1S("div")));
 
         QWebElement div = m_root.firstChild();
         div.addClass(QL1S("download"));
 
-        KUrl u = KUrl(item.destUrlString);
+        KUrl u = KUrl(item->destinationUrl());
         QString fName = u.fileName();
         QString dir = QL1S("file://") + u.directory();
         QString file = dir +  QL1C('/') + fName;
@@ -589,11 +598,11 @@ void NewTabPage::downloadsPage()
 
         div.appendInside(QL1S("<strong>") + fName +  QL1S("</strong>"));
         div.appendInside(QL1S(" - "));
-        QString date = KGlobal::locale()->formatDateTime(item.dateTime, KLocale::FancyLongDate);
+        QString date = KGlobal::locale()->formatDateTime(item->dateTime(), KLocale::FancyLongDate);
         div.appendInside(QL1S("<em>") + date +  QL1S("</em>"));
         div.appendInside(QL1S("<br />"));
 
-        div.appendInside(QL1S("<a href=") + item.srcUrlString +  QL1C('>') + item.srcUrlString +  QL1S("</a>"));
+        div.appendInside(QL1S("<a href=") + item->originUrl() +  QL1C('>') + item->originUrl() +  QL1S("</a>"));
         div.appendInside(QL1S("<br />"));
 
         div.appendInside(markup(QL1S("a")));
@@ -618,3 +627,9 @@ QWebElement NewTabPage::createLinkItem(const QString &title, const QString &urlS
     nav.findFirst(QL1S("span")).appendInside(title);
     return nav;
 }
+
+void NewTabPage::updateWindowIcon()
+{
+    int currentIndex = rApp->mainWindow()->mainView()->currentIndex();
+    rApp->mainWindow()->changeWindowIcon(currentIndex);
+}
\ No newline at end of file
