@@ -68,7 +68,7 @@ static KFileItemList sortFileList(const KFileItemList &list)
     KFileItemList orderedList, dirList, fileList;
 
     // order dirs before files..
-    Q_FOREACH(const KFileItem &item, list)
+    Q_FOREACH(const KFileItem & item, list)
     {
         if (item.isDir())
             dirList << item;
@@ -119,7 +119,6 @@ bool ProtocolHandler::preHandling(const QNetworkRequest &request, QWebFrame *fra
                 return false;
         }
 
-        kDebug() << "EVALUATING JAVASCRIPT...";
         QVariant result = frame->evaluateJavaScript(scriptSource);
         return true;
     }
@@ -176,12 +175,9 @@ bool ProtocolHandler::preHandling(const QNetworkRequest &request, QWebFrame *fra
     // let webkit try to load a known (or missing) protocol...
     if (KProtocolInfo::isKnownProtocol(_url))
         return false;
-    
+
     // Error Message, for those protocols we cannot handle
-        // FIXME change this sentence AFTER STRING FREEZE to: "rekonq doesn't know how to handle this protocol: _url.protocol()" 
-    KMessageBox::error(rApp->mainWindow(), i18nc("@info",
-                       "rekonq cannot handle this URL. \
-                        Please use an appropriate application to open it."));
+    KMessageBox::error(rApp->mainWindow(), i18nc("@info", "rekonq does not know how to handle this protocol: %1", _url.protocol()));
 
     return true;
 }
@@ -192,8 +188,6 @@ bool ProtocolHandler::postHandling(const QNetworkRequest &request, QWebFrame *fr
     _url = request.url();
     _frame = frame;
 
-    kDebug() << "URL PROTOCOL: " << _url;
-
     // "http(s)" (fast) handling
     if (_url.protocol() == QL1S("http") || _url.protocol() == QL1S("https"))
         return false;
@@ -231,6 +225,14 @@ bool ProtocolHandler::postHandling(const QNetworkRequest &request, QWebFrame *fr
         }
     }
 
+    // we cannot handle this protocol in any way.
+    // Try KRunning it...
+    if (KProtocolInfo::isKnownProtocol(_url))
+    {
+        (void)new KRun(_url, rApp->mainWindow(), 0, _url.isLocalFile());
+        return true;
+    }
+
     return false;
 }
 
@@ -293,7 +295,7 @@ QString ProtocolHandler::dirHandling(const KFileItemList &list)
     msg += "<tr><th align=\"left\">" + i18n("Name") + "</th><th>" + i18n("Size") + "</th><th>" + i18n("Last Modified") + "</th></tr>";
 
     KFileItemList orderedList = sortFileList(list);
-    Q_FOREACH(const KFileItem &item, orderedList)
+    Q_FOREACH(const KFileItem & item, orderedList)
     {
         msg += "<tr>";
         QString fullPath = item.url().prettyUrl();
@@ -309,7 +311,7 @@ QString ProtocolHandler::dirHandling(const KFileItemList &list)
         msg += "<td align=\"right\">";
         if (item.isFile())
         {
-            msg += QString::number(item.size() / 1024) + " KB";
+            msg += KGlobal::locale()->formatByteSize(item.size(), 1);
         }
         msg += "</td>";
 
@@ -336,7 +338,6 @@ void ProtocolHandler::slotMostLocalUrlResult(KJob *job)
     if (job->error())
     {
         // TODO
-        kDebug() << "error";
     }
     else
     {
@@ -361,8 +362,6 @@ void ProtocolHandler::slotMostLocalUrlResult(KJob *job)
  */
 void ProtocolHandler::abpHandling()
 {
-    kDebug() << "handling abp url: " << _url;
-
     QString path = _url.path();
     if (path != QL1S("subscribe"))
         return;
