@@ -42,11 +42,11 @@
 #include "webtab.h"
 #include "websnap.h"
 #include "tabhighlighteffect.h"
+#include "tabpreviewpopup.h"
 
 // KDE Includes
 #include <KActionMenu>
 #include <KMenu>
-#include <KPassivePopup>
 #include <KToolBar>
 #include <KColorScheme>
 
@@ -59,10 +59,6 @@
 #include <QStyleOptionFrameV3>
 
 
-#define BASE_WIDTH_DIVISOR    4
-#define MIN_WIDTH_DIVISOR     8
-
-
 static inline QByteArray highlightPropertyName(int index)
 {
     return QByteArray("hAnim").append(QByteArray::number(index));
@@ -87,11 +83,12 @@ TabBar::TabBar(QWidget *parent)
 
     connect(this, SIGNAL(contextMenu(int, const QPoint &)), this, SLOT(contextMenu(int, const QPoint &)));
     connect(this, SIGNAL(emptyAreaContextMenu(const QPoint &)), this, SLOT(emptyAreaContextMenu(const QPoint &)));
+    connect(this, SIGNAL(tabMoved(int, int)), this, SLOT(tabMoved(int, int)));
 
     connect(m_animationMapper, SIGNAL(mapped(int)), this, SLOT(removeAnimation(int)));
     setGraphicsEffect(m_tabHighlightEffect);
 
-    setAnimatedTabHighlighting( ReKonfig::animatedTabHighlighting() );
+    setAnimatedTabHighlighting(ReKonfig::animatedTabHighlighting());
 }
 
 
@@ -101,11 +98,11 @@ QSize TabBar::tabSizeHint(int index) const
 
     int buttonSize = view->addTabButton()->size().width();
     int tabBarWidth = view->size().width() - buttonSize;
-    int baseWidth =  view->sizeHint().width() / BASE_WIDTH_DIVISOR;
-    int minWidth =  view->sizeHint().width() / MIN_WIDTH_DIVISOR;
+    int baseWidth =  view->sizeHint().width() / baseWidthDivisor;
+    int minWidth =  view->sizeHint().width() / minWidthDivisor;
 
     int w;
-    if (baseWidth*count() < tabBarWidth)
+    if (baseWidth * count() < tabBarWidth)
     {
         w = baseWidth;
     }
@@ -185,22 +182,23 @@ void TabBar::showTabPreview()
     if (indexedTab->isPageLoading())
         return;
 
-    int w = tabSizeHint(m_currentTabPreviewIndex).width();
-    int h = w * ((0.0 + currentTab->height()) / currentTab->width());
+    int w = (mv->sizeHint().width() / baseWidthDivisor);
 
-    m_previewPopup = new KPassivePopup(this);
-    m_previewPopup.data()->setFrameShape(QFrame::StyledPanel);
-    m_previewPopup.data()->setFrameShadow(QFrame::Plain);
-    m_previewPopup.data()->setFixedSize(w, h);
+    m_previewPopup = new TabPreviewPopup(indexedTab , this);
 
-    QLabel *l = new QLabel();
-    l->setPixmap(WebSnap::renderTabPreview(*indexedTab->page(), w, h));
+    int tabBarWidth = mv->size().width();
+    int leftIndex = tabRect(m_currentTabPreviewIndex).x() + (tabRect(m_currentTabPreviewIndex).width() - w) / 2;
 
-    m_previewPopup.data()->setView(l);
-    m_previewPopup.data()->layout()->setAlignment(Qt::AlignTop);
-    m_previewPopup.data()->layout()->setMargin(0);
+    if (leftIndex < 0)
+    {
+        leftIndex = 0;
+    }
+    else if (leftIndex + w > tabBarWidth)
+    {
+        leftIndex = tabBarWidth - w;
+    }
 
-    QPoint pos(tabRect(m_currentTabPreviewIndex).x() , tabRect(m_currentTabPreviewIndex).y() + tabRect(m_currentTabPreviewIndex).height());
+    QPoint pos(leftIndex, tabRect(m_currentTabPreviewIndex).y() + tabRect(m_currentTabPreviewIndex).height());
     m_previewPopup.data()->show(mapToGlobal(pos));
 }
 
@@ -303,6 +301,13 @@ void TabBar::mousePressEvent(QMouseEvent *event)
 }
 
 
+void TabBar::tabMoved(int, int)
+{
+    MainView *mv = qobject_cast<MainView *>(parent());
+    QTimer::singleShot(200, mv, SIGNAL(tabsChanged()));
+}
+
+
 void TabBar::contextMenu(int tab, const QPoint &pos)
 {
     setupHistoryActions();
@@ -390,10 +395,10 @@ void TabBar::setupHistoryActions()
     if (!isEnabled)
         return;
 
-    Q_FOREACH(const HistoryItem &item, mv->recentlyClosedTabs())
+    Q_FOREACH(const TabHistory & item, mv->recentlyClosedTabs())
     {
         KAction *a = new KAction(rApp->iconManager()->iconForUrl(item.url), item.title, this);
-        a->setData(item.url);
+        a->setData(item.history);
         connect(a, SIGNAL(triggered()), mv, SLOT(openClosedTab()));
         am->addAction(a);
     }
@@ -432,7 +437,7 @@ void TabBar::setTabHighlighted(int index)
             m_animationMapper->setMapping(anim, index);
             connect(anim, SIGNAL(finished()), m_animationMapper, SLOT(map()));
         }
- 
+
         setTabTextColor(index, highlightColor);
     }
 }
