@@ -41,7 +41,7 @@
 #include <QSignalMapper>
 
 // Forward Declarations
-class KPassivePopup;
+class TabPreviewPopup;
 class TabHighlightEffect;
 class QPropertyAnimation;
 
@@ -63,6 +63,10 @@ public:
     QRect tabTextRect(int index);
     void setAnimatedTabHighlighting(bool enabled);
 
+    //constants
+    static const int baseWidthDivisor = 4;
+    static const int minWidthDivisor = 8;
+
 signals:
     void cloneTab(int index);
     void closeTab(int index);
@@ -94,6 +98,7 @@ private slots:
 
     void contextMenu(int, const QPoint &);
     void emptyAreaContextMenu(const QPoint &);
+    void tabMoved(int, int);
 
     void showTabPreview();
 
@@ -108,7 +113,7 @@ private:
      */
     int m_actualIndex;
 
-    QWeakPointer<KPassivePopup> m_previewPopup;
+    QWeakPointer<TabPreviewPopup> m_previewPopup;
 
     /**
      * the index of the tab preview shown
