@@ -21,10 +21,6 @@
 
 #include <qtest_kde.h>
 
-#include <QtCore>
-#include <QtGui>
-#include <QtTest>
-
 #include "walletbar.h"
 
 
