@@ -46,8 +46,8 @@
 
 
 BookmarkWidget::BookmarkWidget(const KBookmark &bookmark, QWidget *parent)
-        : QMenu(parent)
-        , m_bookmark(new KBookmark(bookmark))
+    : QMenu(parent)
+    , m_bookmark(new KBookmark(bookmark))
 {
     setAttribute(Qt::WA_DeleteOnClose);
     setFixedWidth(350);
@@ -55,17 +55,16 @@ BookmarkWidget::BookmarkWidget(const KBookmark &bookmark, QWidget *parent)
     QFormLayout *layout = new QFormLayout(this);
 
     // Bookmark icon
-    QHBoxLayout *hLayout = new QHBoxLayout(this);
     QLabel *bookmarkIcon = new QLabel(this);
     bookmarkIcon->setPixmap(KIcon("bookmarks").pixmap(32, 32));
-    hLayout->setSpacing(10);
-    hLayout->addWidget(bookmarkIcon);
 
     // Title
     QVBoxLayout *vLayout = new QVBoxLayout(this);
     QLabel *bookmarkInfo = new QLabel(this);
-    bookmarkInfo->setText(i18n("<h4>Edit this Bookmark</h4>"));
-    bookmarkInfo->setAlignment(Qt::AlignCenter);
+    bookmarkInfo->setText(i18n("Edit this Bookmark"));
+    QFont f = bookmarkInfo->font();
+    f.setBold(true);
+    bookmarkInfo->setFont(f);
     vLayout->addWidget(bookmarkInfo);
 
     // Remove button
@@ -74,8 +73,7 @@ BookmarkWidget::BookmarkWidget(const KBookmark &bookmark, QWidget *parent)
     connect(removeButton, SIGNAL(clicked()), this, SLOT(removeBookmark()));
     vLayout->addWidget(removeButton);
 
-    hLayout->addLayout(vLayout);
-    layout->addItem(hLayout);
+    layout->addRow(bookmarkIcon, vLayout);
 
     // Bookmark name
     QLabel *nameLabel = new QLabel(this);
