@@ -60,9 +60,9 @@
 
 
 ListItem::ListItem(const UrlSearchItem &item, QWidget *parent)
-        : QWidget(parent)
-        , m_option()
-        , m_url(item.url)
+    : QWidget(parent)
+    , m_option()
+    , m_url(item.url)
 {
     m_option.initFrom(this);
     m_option.direction = Qt::LeftToRight;
@@ -150,7 +150,7 @@ QString ListItem::text()
 
 void ListItem::nextItemSubChoice()
 {
-    //will be override
+    // will be override
 }
 
 
@@ -158,7 +158,7 @@ void ListItem::nextItemSubChoice()
 
 
 TypeIconLabel::TypeIconLabel(int type, QWidget *parent)
-        : QLabel(parent)
+    : QLabel(parent)
 {
     setMinimumWidth(40);
     QHBoxLayout *hLayout = new QHBoxLayout;
@@ -193,7 +193,7 @@ QLabel *TypeIconLabel::getIcon(QString icon)
 
 
 IconLabel::IconLabel(const QString &icon, QWidget *parent)
-        : QLabel(parent)
+    : QLabel(parent)
 {
     QPixmap pixmapIcon = rApp->iconManager()->iconForUrl(KUrl(icon)).pixmap(16);
     setFixedSize(16, 16);
@@ -202,7 +202,7 @@ IconLabel::IconLabel(const QString &icon, QWidget *parent)
 
 
 IconLabel::IconLabel(const KIcon &icon, QWidget *parent)
-        : QLabel(parent)
+    : QLabel(parent)
 {
     QPixmap pixmapIcon = icon.pixmap(16);
     setFixedSize(16, 16);
@@ -217,7 +217,7 @@ static QString highlightWordsInText(const QString &text, const QStringList &word
 {
     QString ret = text;
     QBitArray boldSections(ret.size());
-    foreach(const QString &wordToPointOut, words)
+    foreach(const QString & wordToPointOut, words)
     {
         int index = ret.indexOf(wordToPointOut, 0, Qt::CaseInsensitive);
         while (index > -1)
@@ -227,10 +227,10 @@ static QString highlightWordsInText(const QString &text, const QStringList &word
         }
     }
 
-    
+
     if (boldSections.isEmpty())
         return ret;
-    
+
     int numSections = 0;
     for (int i = 0; i < boldSections.size() - 1; ++i)
     {
@@ -262,7 +262,7 @@ static QString highlightWordsInText(const QString &text, const QStringList &word
 
 
 TextLabel::TextLabel(const QString &text, const QString &textToPointOut, QWidget *parent)
-        : QLabel(parent)
+    : QLabel(parent)
 {
     setTextFormat(Qt::RichText);
     setMouseTracking(false);
@@ -281,7 +281,7 @@ TextLabel::TextLabel(const QString &text, const QString &textToPointOut, QWidget
 
 
 TextLabel::TextLabel(QWidget *parent)
-        : QLabel(parent)
+    : QLabel(parent)
 {
     setTextFormat(Qt::RichText);
     setMouseTracking(false);
@@ -299,7 +299,7 @@ void TextLabel::setEngineText(const QString &engine, const QString &text)
 
 
 DescriptionLabel::DescriptionLabel(const QString &text, QWidget *parent)
-        : QLabel(parent)
+    : QLabel(parent)
 {
     QString t = text;
     const bool wasItalic = t.startsWith(QL1S("<i>"));
@@ -319,7 +319,7 @@ DescriptionLabel::DescriptionLabel(const QString &text, QWidget *parent)
 
 
 PreviewListItem::PreviewListItem(const UrlSearchItem &item, const QString &text, QWidget *parent)
-        : ListItem(item, parent)
+    : ListItem(item, parent)
 {
     QHBoxLayout *hLayout = new QHBoxLayout;
     hLayout->setSpacing(4);
@@ -357,7 +357,7 @@ PreviewListItem::PreviewListItem(const UrlSearchItem &item, const QString &text,
 
 
 PreviewLabel::PreviewLabel(const QString &url, int width, int height, QWidget *parent)
-        : QLabel(parent)
+    : QLabel(parent)
 {
     setFixedSize(width, height);
     setFrameStyle(QFrame::StyledPanel | QFrame::Raised);
@@ -376,8 +376,8 @@ PreviewLabel::PreviewLabel(const QString &url, int width, int height, QWidget *p
 
 
 ImageLabel::ImageLabel(const QString &url, int width, int height, QWidget *parent)
-        : QLabel(parent),
-        m_url(url)
+    : QLabel(parent),
+      m_url(url)
 {
     setFixedSize(width, height);
     if (WebSnap::existsImage(KUrl(url)))
@@ -418,8 +418,8 @@ void ImageLabel::slotResult(KJob *)
 
 
 SearchListItem::SearchListItem(const UrlSearchItem &item, const QString &text, QWidget *parent)
-        : ListItem(item, parent)
-        , m_text(text)
+    : ListItem(item, parent)
+    , m_text(text)
 {
     m_iconLabel = new IconLabel(SearchEngine::buildQuery(UrlResolver::searchEngine(), ""), this);
     m_titleLabel = new TextLabel(this);
@@ -464,7 +464,7 @@ void SearchListItem::nextItemSubChoice()
 
 
 EngineBar::EngineBar(KService::Ptr selectedEngine, QWidget *parent)
-        : KToolBar(parent)
+    : KToolBar(parent)
 {
     setIconSize(QSize(16, 16));
     setToolButtonStyle(Qt::ToolButtonIconOnly);
@@ -475,7 +475,7 @@ EngineBar::EngineBar(KService::Ptr selectedEngine, QWidget *parent)
     if (SearchEngine::defaultEngine().isNull())
         return;
     m_engineGroup->addAction(newEngineAction(SearchEngine::defaultEngine(), selectedEngine));
-    foreach(KService::Ptr engine, SearchEngine::favorites())
+    foreach(const KService::Ptr & engine, SearchEngine::favorites())
     {
         if (engine->desktopEntryName() != SearchEngine::defaultEngine()->desktopEntryName())
         {
@@ -535,8 +535,8 @@ void EngineBar::selectNextEngine()
 
 
 SuggestionListItem::SuggestionListItem(const UrlSearchItem &item, const QString &text, QWidget *parent)
-        : ListItem(item, parent)
-        , m_text(item.title)
+    : ListItem(item, parent)
+    , m_text(item.title)
 {
     QHBoxLayout *hLayout = new QHBoxLayout;
     hLayout->setSpacing(4);
@@ -559,8 +559,8 @@ QString SuggestionListItem::text()
 
 
 VisualSuggestionListItem::VisualSuggestionListItem(const UrlSearchItem &item, const QString &text, QWidget *parent)
-        : ListItem(item, parent)
-        , m_text(item.title)
+    : ListItem(item, parent)
+    , m_text(item.title)
 {
 
     QHBoxLayout *hLayout = new QHBoxLayout;
@@ -605,7 +605,7 @@ QString VisualSuggestionListItem::text()
 
 
 BrowseListItem::BrowseListItem(const UrlSearchItem &item, const QString &text, QWidget *parent)
-        : ListItem(item, parent)
+    : ListItem(item, parent)
 {
     QString url = text;
 
@@ -629,41 +629,33 @@ ListItem *ListItemFactory::create(const UrlSearchItem &item, const QString &text
 {
     if (item.type & UrlSearchItem::Search)
     {
-        kDebug() << "Search";
         return new SearchListItem(item, text, parent);
     }
 
     if (item.type & UrlSearchItem::Browse)
     {
-        kDebug() << "Browse";
         return new BrowseListItem(item, text, parent);
     }
 
     if (item.type & UrlSearchItem::History)
     {
-        kDebug() << "History";
         return new PreviewListItem(item, text, parent);
     }
 
     if (item.type & UrlSearchItem::Bookmark)
     {
-        kDebug() << "Bookmark";
         return new PreviewListItem(item, text, parent);
     }
 
     if (item.type & UrlSearchItem::Suggestion)
     {
-        kDebug() << "ITEM URL: " << item.url;
         if (item.description.isEmpty())
         {
-            kDebug() << "Suggestion";
             return new SuggestionListItem(item, text, parent);
         }
 
-        kDebug() << "Visual Suggestion";
         return new VisualSuggestionListItem(item, text, parent);
     }
 
-    kDebug() << "Undefined";
     return new PreviewListItem(item, text, parent);
 }
