@@ -52,18 +52,20 @@
 
 
 RSSWidget::RSSWidget(const QMap< KUrl, QString > &map, QWidget *parent)
-        : QMenu(parent)
-        , m_map(map)
+    : QMenu(parent)
+    , m_map(map)
 {
     setAttribute(Qt::WA_DeleteOnClose);
-    setMinimumWidth(250);
+    setFixedWidth(350);
 
     QFormLayout *layout = new QFormLayout(this);
 
     // Title
     QLabel *title = new QLabel(this);
-    title->setText(i18n("<h4>Subscribe to RSS Feeds</h4>"));
-    title->setAlignment(Qt::AlignCenter);
+    title->setText(i18n("Subscribe to RSS Feeds"));
+    QFont f = title->font();
+    f.setBold(true);
+    title->setFont(f);
     layout->addRow(title);
 
     // Agregators
@@ -85,7 +87,7 @@ RSSWidget::RSSWidget(const QMap< KUrl, QString > &map, QWidget *parent)
     m_feeds = new KComboBox(this);
     m_feeds->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
 
-    foreach(const QString &title, m_map)
+    foreach(const QString & title, m_map)
     {
         m_feeds->addItem(title);
     }
