@@ -45,22 +45,26 @@
 #include "bookmarkowner.h"
 #include "bookmarkwidget.h"
 #include "iconmanager.h"
+#include "favoritewidget.h"
+#include "searchengine.h"
 
 // KDE Includes
 #include <KCompletionBox>
 #include <KStandardDirs>
 #include <KColorScheme>
+#include <KMenu>
 
 // Qt Includes
 #include <QtGui/QPainter>
 #include <QtGui/QPaintEvent>
 #include <QtGui/QPalette>
 #include <QtGui/QVBoxLayout>
+#include <QClipboard>
 
 
 
 IconButton::IconButton(QWidget *parent)
-        : QToolButton(parent)
+    : QToolButton(parent)
 {
     setToolButtonStyle(Qt::ToolButtonIconOnly);
     setStyleSheet("IconButton { background-color:transparent; border: none; padding: 0px}");
@@ -78,10 +82,10 @@ void IconButton::mouseReleaseEvent(QMouseEvent* event)
 
 
 UrlBar::UrlBar(QWidget *parent)
-        : KLineEdit(parent)
-        , _tab(0)
-        , _icon(new IconButton(this))
-        , _suggestionTimer(new QTimer(this))
+    : KLineEdit(parent)
+    , _tab(0)
+    , _icon(new IconButton(this))
+    , _suggestionTimer(new QTimer(this))
 {
     // initial style
     setStyleSheet(QString("UrlBar { padding: 0 0 0 %1px;} ").arg(_icon->sizeHint().width()));
@@ -114,7 +118,8 @@ UrlBar::UrlBar(QWidget *parent)
     connect(_tab->view(), SIGNAL(iconChanged()), this, SLOT(refreshFavicon()));
 
     // search icon
-    connect(rApp->opensearchManager(), SIGNAL(openSearchEngineAdded(const QString &, const QString &, const QString &)), this, SLOT(updateRightIcons()));
+    connect(rApp->opensearchManager(), SIGNAL(openSearchEngineAdded(const QString &, const QString &, const QString &)),
+            this, SLOT(updateRightIcons()));
 
     _suggestionTimer->setSingleShot(true);
     connect(_suggestionTimer, SIGNAL(timeout()), this, SLOT(suggest()));
@@ -178,11 +183,14 @@ void UrlBar::paintEvent(QPaintEvent *event)
     QPalette p = palette();
 
     int progr = _tab->progress();
-    if (progr == 0)
+    if (progr == 0 || progr == 100)
     {
         if (_tab->url().scheme() == QL1S("https"))
         {
-            backgroundColor = colorScheme.background(KColorScheme::NeutralBackground).color();  // light yellow
+            backgroundColor = _tab->page()->hasSslValid()
+                              ? colorScheme.background(KColorScheme::PositiveBackground).color()
+                              : colorScheme.background(KColorScheme::NegativeBackground).color();
+
             foregroundColor = colorScheme.foreground(KColorScheme::NormalText).color();
         }
         p.setBrush(QPalette::Base, backgroundColor);
@@ -285,6 +293,9 @@ void UrlBar::keyPressEvent(QKeyEvent *event)
     else if (event->key() == Qt::Key_Escape)
     {
         clearFocus();
+        if (text() != rApp->mainWindow()->currentTab()->view()->url().toString()
+                && !rApp->mainWindow()->currentTab()->view()->url().toString().startsWith("about"))
+            setText(rApp->mainWindow()->currentTab()->view()->url().toString());
         event->accept();
     }
 
@@ -302,6 +313,20 @@ void UrlBar::focusInEvent(QFocusEvent *event)
 
 void UrlBar::dropEvent(QDropEvent *event)
 {
+    // handles only plain-text with url format
+    if (event->mimeData()->hasFormat("text/plain") && event->source() != this)
+    {
+        QUrl url = QUrl::fromUserInput(event->mimeData()->data("text/plain"));
+
+        if (url.isValid())
+        {
+            setQUrl(url);
+            activated(text());
+            return;
+        }
+    }
+
+    // handles everything else
     KLineEdit::dropEvent(event);
     activated(text());
 }
@@ -315,9 +340,16 @@ void UrlBar::loadFinished()
         return;
     }
 
+    // show Favorite Icon
+    if (ReKonfig::previewUrls().contains(_tab->url().url()))
+    {
+        IconButton *bt = addRightIcon(UrlBar::Favorite);
+        connect(bt, SIGNAL(clicked(QPoint)), this, SLOT(showFavoriteDialog(QPoint)));
+    }
+
     // show bookmark info
     IconButton *bt = addRightIcon(UrlBar::BK);
-    connect(bt, SIGNAL(clicked(const QPoint &)), this, SLOT(showBookmarkInfo(const QPoint &)));
+    connect(bt, SIGNAL(clicked(QPoint)), this, SLOT(showBookmarkInfo(QPoint)));
 
     // show KGet downloads??
     if (!KStandardDirs::findExe("kget").isNull() && ReKonfig::kgetList())
@@ -336,6 +368,7 @@ void UrlBar::loadFinished()
     // show SSL
     if (_tab->url().scheme() == QL1S("https"))
     {
+        // NOTE: the choice for the right SSL icon is done in the addRightIcon method
         IconButton *bt = addRightIcon(UrlBar::SSL);
         connect(bt, SIGNAL(clicked(QPoint)), _tab->page(), SLOT(showSSLInfo(QPoint)));
     }
@@ -350,30 +383,28 @@ void UrlBar::loadFinished()
     // we need to update urlbar after the right icon settings
     // removing this code (where setStyleSheet automatically calls update) needs adding again
     // an update call
-    kDebug() << "resetting stylesheet";
     int rightIconWidth = 25 * (_rightIconsList.count());
     setStyleSheet(QString("UrlBar { padding: 0 %2px 0 %1px;} ").arg(_icon->sizeHint().width()).arg(rightIconWidth));
 }
 
 
-void UrlBar::showBookmarkInfo(const QPoint &pos)
+void UrlBar::showBookmarkDialog()
+{
+    showBookmarkInfo(QCursor::pos());
+}
+
+
+void UrlBar::showBookmarkInfo(QPoint pos)
 {
     if (_tab->url().scheme() == QL1S("about"))
         return;
 
     KBookmark bookmark = rApp->bookmarkProvider()->bookmarkForUrl(_tab->url());
 
-    IconButton *bt = qobject_cast<IconButton *>(this->sender());
-    if (!bt)
-        return;
-
     if (bookmark.isNull())
     {
         bookmark = rApp->bookmarkProvider()->bookmarkOwner()->bookmarkCurrentPage();
-
-        // set bk icon
-        bt->setIcon(KIcon("bookmarks"));
-        bt->setToolTip(i18n("Edit this bookmark"));
+        updateRightIcons();
     }
     else
     {
@@ -423,12 +454,77 @@ void UrlBar::activateSuggestions(bool b)
 }
 
 
-void UrlBar::mouseDoubleClickEvent(QMouseEvent *)
+void UrlBar::mouseDoubleClickEvent(QMouseEvent *event)
 {
+    Q_UNUSED(event);
+
     selectAll();
 }
 
 
+void UrlBar::contextMenuEvent(QContextMenuEvent* event)
+{
+    KMenu menu;
+    const bool clipboardFilled = !rApp->clipboard()->text().isEmpty();
+
+    // Cut
+    KAction *a = KStandardAction::cut(this, SLOT(cut()), this);
+    a->setEnabled(hasSelectedText());
+    menu.addAction(a);
+
+    // Copy
+    a = KStandardAction::copy(this, SLOT(copy()), this);
+    a->setEnabled(hasSelectedText());
+    menu.addAction(a);
+
+    // Paste
+    a = KStandardAction::paste(this, SLOT(paste()), this);
+    a->setEnabled(clipboardFilled);
+    menu.addAction(a);
+
+    // Paste & Go
+    const QString clipboardText = rApp->clipboard()->text();
+    if (isValidURL(clipboardText) || clipboardText.isEmpty())
+    {
+        a = new KAction(i18n("Paste && Go"), this);
+        connect(a, SIGNAL(triggered(bool)), this, SLOT(pasteAndGo()));
+    }
+    else
+    {
+        a = new KAction(i18n("Paste && Search"), this);
+        connect(a, SIGNAL(triggered(bool)), this, SLOT(pasteAndSearch()));
+    }
+    a->setEnabled(clipboardFilled);
+    menu.addAction(a);
+
+    // Delete
+    a = new KAction(KIcon("edit-delete"), i18n("Delete"), this);
+    connect(a, SIGNAL(triggered(bool)), this, SLOT(delSlot()));
+    a->setEnabled(hasSelectedText());
+    menu.addAction(a);
+
+    menu.addSeparator();
+
+    // Select All
+    a = KStandardAction::selectAll(this, SLOT(selectAll()), this);
+    a->setEnabled(!text().isEmpty());
+    menu.addAction(a);
+
+    menu.exec(event->globalPos());
+}
+
+
+bool UrlBar::isValidURL(QString url)
+{
+    bool isValid = false;
+    if (url.startsWith("http://") || url.startsWith("https://") || url.startsWith("ftp://"))
+        url = url.remove(QRegExp("(http|https|ftp)://"));
+    if (url.contains('.') && url.indexOf('.') > 0 && url.indexOf('.') < url.length() && !url.trimmed().contains(" ") && QUrl::fromUserInput(url).isValid())
+        isValid = true;
+    return isValid;
+}
+
+
 IconButton *UrlBar::addRightIcon(UrlBar::icon ic)
 {
     IconButton *rightIcon = new IconButton(this);
@@ -444,7 +540,9 @@ IconButton *UrlBar::addRightIcon(UrlBar::icon ic)
         rightIcon->setToolTip(i18n("List all available RSS feeds"));
         break;
     case UrlBar::SSL:
-        rightIcon->setIcon(KIcon("object-locked"));
+        _tab->page()->hasSslValid()
+        ? rightIcon->setIcon(KIcon("object-locked"))
+        : rightIcon->setIcon(KIcon("object-unlocked"));
         rightIcon->setToolTip(i18n("Show SSL Info"));
         break;
     case UrlBar::BK:
@@ -458,6 +556,8 @@ IconButton *UrlBar::addRightIcon(UrlBar::icon ic)
             rightIcon->setIcon(KIcon("bookmarks"));
             rightIcon->setToolTip(i18n("Edit this bookmark"));
         }
+        rightIcon->setContextMenuPolicy(Qt::CustomContextMenu);
+        connect(rightIcon, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(bookmarkContextMenu(QPoint)));
         break;
     case UrlBar::SearchEngine:
     {
@@ -470,15 +570,19 @@ IconButton *UrlBar::addRightIcon(UrlBar::icon ic)
         rightIcon->setToolTip(i18n("Add search engine"));
         break;
     }
+    case UrlBar::Favorite:
+        rightIcon->setIcon(KIcon("emblem-favorite"));
+        rightIcon->setToolTip(i18n("Remove from favorite"));
+        break;
     default:
-        kDebug() << "ERROR.. default non extant case!!";
+        ASSERT_NOT_REACHED("ERROR.. default non extant case!!");
         break;
     }
 
     _rightIconsList << rightIcon;
     int iconsCount = _rightIconsList.count();
     int iconHeight = (height() - 18) / 2;
-    rightIcon->move(width() - 23*iconsCount, iconHeight);
+    rightIcon->move(width() - 23 * iconsCount, iconHeight);
     rightIcon->show();
 
     return rightIcon;
@@ -503,7 +607,7 @@ void UrlBar::resizeEvent(QResizeEvent *event)
     for (int i = 0; i < iconsCount; ++i)
     {
         IconButton *bt = _rightIconsList.at(i);
-        bt->move(w - 25*(i + 1), newHeight);
+        bt->move(w - 25 * (i + 1), newHeight);
     }
 
     KLineEdit::resizeEvent(event);
@@ -547,3 +651,81 @@ void UrlBar::refreshFavicon()
     }
     _icon->setIcon(rApp->iconManager()->iconForUrl(u));
 }
+
+
+void UrlBar::showFavoriteDialog(QPoint pos)
+{
+    if (_tab->url().scheme() == QL1S("about"))
+        return;
+
+    IconButton *bt = qobject_cast<IconButton *>(this->sender());
+    if (!bt)
+        return;
+
+    FavoriteWidget *widget = new FavoriteWidget(_tab, window());
+    connect(widget, SIGNAL(updateIcon()), this, SLOT(updateRightIcons()));
+    widget->showAt(pos);
+}
+
+
+void UrlBar::bookmarkContextMenu(QPoint pos)
+{
+    Q_UNUSED(pos);
+
+    KMenu menu(this);
+    QAction *qa;
+
+    if (!rApp->bookmarkProvider()->bookmarkForUrl(_tab->url()).isNull())
+    {
+        qa = new KAction(KIcon("bookmarks"), i18n("Edit Bookmark"), this);
+        connect(qa, SIGNAL(triggered(bool)), this, SLOT(showBookmarkDialog()));
+        menu.addAction(qa);
+    }
+
+    if (!ReKonfig::previewUrls().contains(_tab->url().url()))
+    {
+        qa = new KAction(KIcon("emblem-favorite"), i18n("Add to favorite"), this);
+        connect(qa, SIGNAL(triggered(bool)), this, SLOT(addFavorite()));
+        menu.addAction(qa);
+    }
+
+    menu.exec(QCursor::pos());
+}
+
+
+
+void UrlBar::addFavorite()
+{
+    if (ReKonfig::previewUrls().contains(_tab->url().url()))
+        return;
+
+    QStringList urls = ReKonfig::previewUrls();
+    urls << _tab->url().url();
+    ReKonfig::setPreviewUrls(urls);
+
+    QStringList titles = ReKonfig::previewNames();
+    titles << _tab->view()->title();
+    ReKonfig::setPreviewNames(titles);
+
+    updateRightIcons();
+}
+
+
+void UrlBar::pasteAndGo()
+{
+    activated(rApp->clipboard()->text());
+}
+
+
+void UrlBar::pasteAndSearch()
+{
+    KService::Ptr defaultEngine = SearchEngine::defaultEngine();
+    if (defaultEngine)
+        activated(KUrl(SearchEngine::buildQuery(defaultEngine, rApp->clipboard()->text())));
+}
+
+
+void UrlBar::delSlot()
+{
+    del();
+}
