@@ -40,6 +40,7 @@
 // Local Includes
 #include "adblockmanager.h"
 #include "application.h"
+#include "downloadmanager.h"
 #include "iconmanager.h"
 #include "mainview.h"
 #include "mainwindow.h"
@@ -48,8 +49,9 @@
 #include "urlbar.h"
 #include "webpluginfactory.h"
 #include "webtab.h"
-
-#include "sslinfodialog_p.h"
+#include "searchengine.h"
+#include "sslwidget.h"
+#include "sslinfodialog.h"
 
 // KDE Includes
 #include <KIO/Job>
@@ -209,7 +211,7 @@ static bool downloadResource(const KUrl& srcUrl, const KIO::MetaData& metaData =
     while (result == KIO::R_CANCEL && destUrl.isValid());
 
     // Save download history
-    rApp->addDownload(srcUrl.pathOrUrl() , destUrl.pathOrUrl());
+    DownloadItem *item = rApp->downloadManager()->addDownload(srcUrl.pathOrUrl(), destUrl.pathOrUrl());
 
     if (!KStandardDirs::findExe("kget").isNull() && ReKonfig::kgetDownload())
     {
@@ -219,15 +221,24 @@ static bool downloadResource(const KUrl& srcUrl, const KIO::MetaData& metaData =
             KToolInvocation::kdeinitExecWait("kget");
         }
         QDBusInterface kget("org.kde.kget", "/KGet", "org.kde.kget.main");
-        if (kget.isValid())
-        {
-            kget.call("addTransfer", srcUrl.prettyUrl(), destUrl.prettyUrl(), true);
+        if (!kget.isValid())
+            return false;
+
+        QDBusMessage transfer = kget.call(QL1S("addTransfer"), srcUrl.prettyUrl(), destUrl.prettyUrl(), true);
+        if (transfer.arguments().isEmpty())
             return true;
-        }
-        return false;
+
+        const QString transferPath = transfer.arguments().first().toString();
+        item->setKGetTransferDbusPath(transferPath);
+        return true;
     }
 
     KIO::Job *job = KIO::file_copy(srcUrl, destUrl, -1, KIO::Overwrite);
+    if (item)
+    {
+        QObject::connect(job, SIGNAL(percent(KJob *, unsigned long)), item, SLOT(updateProgress(KJob *, unsigned long)));
+        QObject::connect(job, SIGNAL(finished(KJob *)), item, SLOT(onFinished(KJob*)));
+    }
 
     if (!metaData.isEmpty())
         job->setMetaData(metaData);
@@ -236,7 +247,6 @@ static bool downloadResource(const KUrl& srcUrl, const KIO::MetaData& metaData =
     job->addMetaData(QL1S("cache"), QL1S("cache")); // Use entry from cache if available.
     job->uiDelegate()->setAutoErrorHandlingEnabled(true);
     return true;
-
 }
 
 
@@ -244,9 +254,9 @@ static bool downloadResource(const KUrl& srcUrl, const KIO::MetaData& metaData =
 
 
 WebPage::WebPage(QWidget *parent)
-        : KWebPage(parent, KWalletIntegration)
-        , _networkAnalyzer(false)
-        , _isOnRekonqPage(false)
+    : KWebPage(parent, KWalletIntegration)
+    , _networkAnalyzer(false)
+    , _isOnRekonqPage(false)
 {
     // ----- handling unsupported content...
     setForwardUnsupportedContent(true);
@@ -254,27 +264,28 @@ WebPage::WebPage(QWidget *parent)
 
     // ----- rekonq Network Manager
     NetworkAccessManager *manager = new NetworkAccessManager(this);
-    manager->setCache(0);   // disable QtWebKit cache to just use KIO one..
 
-    // set cookieJar window ID..
+    // disable QtWebKit cache to just use KIO one..
+    manager->setCache(0);
+
+    // set cookieJar window..
     if (parent && parent->window())
-        manager->setCookieJarWindowId(parent->window()->winId());
+        manager->setWindow(parent->window());
+
+    // set network reply object to emit readyRead when it receives meta data
+    manager->setEmitReadyReadOnMetaDataChange(true);
 
     setNetworkAccessManager(manager);
 
     // activate ssl warnings
-    setSessionMetaData("ssl_activate_warnings", "TRUE");
-
-    // Override the 'Accept' header sent by QtWebKit which favors XML over HTML!
-    // Setting the accept meta-data to null will force kio_http to use its own
-    // default settings for this header.
-    setSessionMetaData(QL1S("accept"), QString());
+    setSessionMetaData(QL1S("ssl_activate_warnings"), QL1S("TRUE"));
 
     // ----- Web Plugin Factory
     setPluginFactory(new WebPluginFactory(this));
 
     // ----- last stuffs
     connect(manager, SIGNAL(finished(QNetworkReply*)), this, SLOT(manageNetworkErrors(QNetworkReply*)));
+
     connect(this, SIGNAL(downloadRequested(const QNetworkRequest &)), this, SLOT(downloadRequest(const QNetworkRequest &)));
     connect(this, SIGNAL(loadStarted()), this, SLOT(loadStarted()));
     connect(this, SIGNAL(loadFinished(bool)), this, SLOT(loadFinished(bool)));
@@ -354,6 +365,7 @@ bool WebPage::acceptNavigationRequest(QWebFrame *frame, const QNetworkRequest &r
             break;
 
         default:
+            ASSERT_NOT_REACHED();
             break;
         }
     }
@@ -384,34 +396,19 @@ void WebPage::handleUnsupportedContent(QNetworkReply *reply)
 {
     Q_ASSERT(reply);
 
-    // Put the job on hold...
-#if KDE_IS_VERSION( 4, 5, 96)
-    kDebug() << "PUT REPLY ON HOLD...";
-    KIO::Integration::AccessManager::putReplyOnHold(reply);
-#else
-    reply->abort();
-#endif
+    if (!reply)
+        return;
 
-    // This is probably needed just in ONE stupid case..
+    // handle protocols WebKit cannot handle...
     if (_protHandler.postHandling(reply->request(), mainFrame()))
     {
-        kDebug() << "POST HANDLING the unsupported...";
         return;
     }
 
     if (reply->error() != QNetworkReply::NoError)
         return;
 
-    // get reply url...
-    KUrl replyUrl = reply->url();
-
-    bool isLocal = replyUrl.isLocalFile();
-    if(isLocal && KProtocolInfo::isKnownProtocol(replyUrl))
-    {
-        kDebug() << "WARNING: launching a new app...";
-        new KRun(replyUrl, rApp->mainWindow());  // No need to delete KRun, it autodeletes itself
-        return;
-    }
+    KIO::Integration::AccessManager::putReplyOnHold(reply);
 
     // Get suggested file name...
     extractSuggestedFileName(reply, _suggestedFileName);
@@ -429,10 +426,11 @@ void WebPage::handleUnsupportedContent(QNetworkReply *reply)
 
     KService::Ptr appService = KMimeTypeTrader::self()->preferredService(_mimeType);
 
+    KUrl replyUrl = reply->url();
+    bool isLocal = replyUrl.isLocalFile();
+
     if (appService.isNull())  // no service can handle this. We can just download it..
     {
-        kDebug() << "no service can handle this. We can just download it..";
-
         isLocal
         ? KMessageBox::sorry(view(), i18n("No service can handle this file."))
         : downloadReply(reply, _suggestedFileName);
@@ -449,14 +447,13 @@ void WebPage::handleUnsupportedContent(QNetworkReply *reply)
         switch (dlg.askEmbedOrSave())
         {
         case KParts::BrowserOpenOrSaveQuestion::Save:
-            kDebug() << "user choice: no services, just download!";
             downloadReply(reply, _suggestedFileName);
             return;
 
         case KParts::BrowserOpenOrSaveQuestion::Cancel:
             return;
 
-        default: // non extant case
+        default: // Can happen when "Open with.." is set and "don't ask again" is checked
             break;
         }
     }
@@ -472,7 +469,6 @@ void WebPage::handleUnsupportedContent(QNetworkReply *reply)
         tempFile.open();
         KUrl destUrl;
         destUrl.setPath(tempFile.fileName());
-        kDebug() << "First save content to" << destUrl;
         KIO::Job *job = KIO::file_copy(_loadingUrl, destUrl, 0600, KIO::Overwrite);
         job->ui()->setWindow(rApp->mainWindow());
         connect(job, SIGNAL(result(KJob *)), this, SLOT(copyToTempFileResult(KJob*)));
@@ -483,8 +479,6 @@ void WebPage::handleUnsupportedContent(QNetworkReply *reply)
     KParts::ReadOnlyPart *pa = KMimeTypeTrader::createPartInstanceFromQuery<KParts::ReadOnlyPart>(_mimeType, view(), this, QString());
     if (pa)
     {
-        kDebug() << "EMBEDDING CONTENT...";
-
         _isOnRekonqPage = true;
 
         WebView *view = qobject_cast<WebView *>(parent());
@@ -501,23 +495,9 @@ void WebPage::handleUnsupportedContent(QNetworkReply *reply)
         // No parts, just app services. Load it!
         // If the app is a KDE one, publish the slave on hold to let it use it.
         // Otherwise, run the app and remove it (the io slave...)
-        if (appService->categories().contains(QL1S("KDE"), Qt::CaseInsensitive))
-        {
-#if KDE_IS_VERSION( 4, 5, 96)
-            KIO::Scheduler::publishSlaveOnHold();
-#endif
-            KRun::run(*appService, replyUrl, 0, false, _suggestedFileName);
-            return;
-        }
         KRun::run(*appService, replyUrl, 0, false, _suggestedFileName);
     }
 
-    // Remove any ioslave that was put on hold...
-#if KDE_IS_VERSION( 4, 5, 96)
-    kDebug() << "REMOVE SLAVES ON HOLD...";
-    KIO::Scheduler::removeSlaveOnHold();
-#endif
-
     return;
 }
 
@@ -526,6 +506,7 @@ void WebPage::loadStarted()
 {
 }
 
+
 void WebPage::loadFinished(bool ok)
 {
     Q_UNUSED(ok);
@@ -537,7 +518,6 @@ void WebPage::loadFinished(bool ok)
     val = group.readEntry(_loadingUrl.host(), QString("10"));
 
     int value = val.toInt();
-    kDebug() << "ZOOM VALUE: " << _loadingUrl.host() << value;
     mainFrame()->setZoomFactor(QVariant(value).toReal() / 10);  // Don't allox max +1 values
 
     // Provide site icon. Can this be moved to loadStarted??
@@ -567,9 +547,11 @@ void WebPage::manageNetworkErrors(QNetworkReply *reply)
 
     QWebFrame* frame = qobject_cast<QWebFrame *>(reply->request().originatingObject());
     const bool isMainFrameRequest = (frame == mainFrame());
+    const bool isLoadingUrlReply = (mainFrame()->url() == reply->url());
 
     if (isMainFrameRequest
             && _sslInfo.isValid()
+            && isLoadingUrlReply
             && !domainSchemeMatch(reply->url(), _sslInfo.url())
        )
     {
@@ -583,7 +565,7 @@ void WebPage::manageNetworkErrors(QNetworkReply *reply)
     {
 
     case QNetworkReply::NoError:                             // no error. Simple :)
-        if (isMainFrameRequest && !_sslInfo.isValid())
+        if (isMainFrameRequest && isLoadingUrlReply && !_sslInfo.isValid())
         {
             // Obtain and set the SSL information if any...
             _sslInfo.restoreFrom(reply->attribute(static_cast<QNetworkRequest::Attribute>(KIO::AccessManager::MetaData)), reply->url());
@@ -600,6 +582,7 @@ void WebPage::manageNetworkErrors(QNetworkReply *reply)
         break;
 
     case QNetworkReply::UnknownNetworkError:                 // unknown network-related error detected
+        kDebug() << "------------------ DO WE REALLY NEED THIS??? --------------------";
         _protHandler.postHandling(reply->request(), frame);
         return;
 
@@ -660,21 +643,31 @@ QString WebPage::errorPage(QNetworkReply *reply)
     iconPath.replace(QL1S("16"), QL1S("128"));
 
     QString msg;
-    msg += "<table>";
-    msg += "<tr><td>";
-    msg += "<img src=\"" + iconPath + "\" />";
-    msg += "</td><td>";
-    msg += "<h1>" + reply->errorString() + "</h1>";
-    msg += "<h2>" + i18nc("%1=an URL, e.g.'kde.org'", "When connecting to: <b>%1</b>", urlString) + "</h2>";
-    msg += "</td></tr></table>";
+    msg += QL1S("<table>");
+    msg += QL1S("<tr><td>");
+    msg += QL1S("<img src=\"") + iconPath + QL1S("\" />");
+    msg += QL1S("</td><td>");
+    msg += QL1S("<h1>") + reply->errorString() + QL1S("</h1>");
+    msg += QL1S("<h2>") + i18nc("%1=an URL, e.g.'kde.org'", "When connecting to: <b>%1</b>", urlString) + QL1S("</h2>");
+    msg += QL1S("</td></tr></table>");
 
-    msg += "<ul><li>" + i18n("Check the address for errors such as <b>ww</b>.kde.org instead of <b>www</b>.kde.org");
-    msg += "</li><li>" + i18n("If the address is correct, try to check the network connection.") + "</li><li>" ;
+    msg += QL1S("<ul><li>") + i18n("Check the address for errors such as <b>ww</b>.kde.org instead of <b>www</b>.kde.org");
+    msg += QL1S("</li><li>") + i18n("If the address is correct, try to check the network connection.") + QL1S("</li><li>") ;
     msg += i18n("If your computer or network is protected by a firewall or proxy, make sure that rekonq is permitted to access the network.");
-    msg += "</li><li>" + i18n("Of course, if rekonq does not work properly, you can always say it is a programmer error ;)");
-    msg += "</li></ul><br/><br/>";
-    msg += "<input type=\"button\" id=\"reloadButton\" onClick=\"document.location.href='" + urlString + "';\" value=\"";
-    msg += i18n("Try Again") + "\" />";
+    msg += QL1S("</li><li>") + i18n("Of course, if rekonq does not work properly, you can always say it is a programmer error ;)");
+    msg += QL1S("</li></ul><br/><br/>");
+    msg += QL1S("<input type=\"button\" id=\"reloadButton\" onClick=\"document.location.href='") + urlString + QL1S("';\" value=\"");
+    msg += i18n("Try Again") + QL1S("\" />");
+
+    //Default SearchEngine
+    KService::Ptr defaultEngine = SearchEngine::defaultEngine();
+
+    if (defaultEngine)
+    {
+        msg += i18n("or");
+        msg += QL1S(" <a href=\"") + SearchEngine::buildQuery(defaultEngine, urlString) + QL1S("\">");
+        msg += i18n("Search with %1", defaultEngine->name()) + QL1S("</a>");
+    }
 
     QString html = QString(QL1S(file.readAll()))
                    .arg(title)
@@ -711,14 +704,14 @@ void WebPage::downloadAllContentsWithKGet(QPoint)
     KUrl relativeUrl;
 
     QWebElementCollection images = mainFrame()->documentElement().findAll("img");
-    foreach(const QWebElement &img, images)
+    foreach(const QWebElement & img, images)
     {
         relativeUrl.setEncodedUrl(img.attribute("src").toUtf8(), KUrl::TolerantMode);
         contents << baseUrl.resolved(relativeUrl).toString();
     }
 
     QWebElementCollection links = mainFrame()->documentElement().findAll("a");
-    foreach(const QWebElement &link, links)
+    foreach(const QWebElement & link, links)
     {
         relativeUrl.setEncodedUrl(link.attribute("href").toUtf8(), KUrl::TolerantMode);
         contents << baseUrl.resolved(relativeUrl).toString();
@@ -736,33 +729,12 @@ void WebPage::downloadAllContentsWithKGet(QPoint)
 }
 
 
-void WebPage::showSSLInfo(QPoint)
+void WebPage::showSSLInfo(QPoint pos)
 {
-    if (_sslInfo.isValid())
-    {
-        QPointer<KSslInfoDialog> dlg = new KSslInfoDialog(view());
-        dlg->setSslInfo(_sslInfo.certificateChain(),
-                        _sslInfo.peerAddress().toString(),
-                        mainFrame()->url().host(),
-                        _sslInfo.protocol(),
-                        _sslInfo.ciphers(),
-                        _sslInfo.usedChiperBits(),
-                        _sslInfo.supportedChiperBits(),
-                        KSslInfoDialog::errorsFromString(_sslInfo.certificateErrors())
-                       );
-
-        dlg->exec();
-        delete dlg;
-
-        return;
-    }
-
     if (mainFrame()->url().scheme() == QL1S("https"))
     {
-        KMessageBox::error(view(),
-                           i18n("The SSL information for this site appears to be corrupt."),
-                           i18nc("Secure Sockets Layer", "SSL")
-                          );
+        SSLWidget *widget = new SSLWidget(mainFrame()->url(), _sslInfo, view());
+        widget->showAt(pos);
     }
     else
     {
@@ -791,3 +763,26 @@ void WebPage::copyToTempFileResult(KJob* job)
     else
         (void)KRun::runUrl(static_cast<KIO::FileCopyJob *>(job)->destUrl(), _mimeType, rApp->mainWindow());
 }
+
+
+bool WebPage::hasSslValid() const
+{
+    QList<QSslCertificate> certList = _sslInfo.certificateChain();
+
+    if (certList.isEmpty())
+        return false;
+
+    Q_FOREACH(const QSslCertificate & cert, certList)
+    {
+        if (!cert.isValid())
+            return false;
+    }
+
+    QList<QStringList> errorList = SslInfoDialog::errorsFromString(_sslInfo.certificateErrors());
+    Q_FOREACH(const QStringList & list, errorList)
+    {
+        if (!list.isEmpty())
+            return false;
+    }
+    return true;
+}
