@@ -27,8 +27,8 @@
 
 // Self Includes
 #include "webtab.h"
-#include "rekonq_defines.h"
 #include "webtab.moc"
+
 // Auto Includes
 #include "rekonq.h"
 
@@ -40,7 +40,9 @@
 #include "webpage.h"
 #include "webshortcutwidget.h"
 #include "application.h"
+#include "sessionmanager.h"
 #include "opensearchmanager.h"
+#include "messagebar.h"
 
 // KDE Includes
 #include <KWebWallet>
@@ -56,11 +58,11 @@
 
 
 WebTab::WebTab(QWidget *parent)
-        : QWidget(parent)
-        , m_webView(new WebView(this))
-        , m_urlBar(new UrlBar(this))
-        , m_progress(0)
-        , m_part(0)
+    : QWidget(parent)
+    , m_webView(new WebView(this))
+    , m_urlBar(new UrlBar(this))
+    , m_progress(0)
+    , m_part(0)
 {
     setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
 
@@ -85,6 +87,9 @@ WebTab::WebTab(QWidget *parent)
     connect(m_webView, SIGNAL(loadProgress(int)), this, SLOT(updateProgress(int)));
     connect(m_webView, SIGNAL(loadFinished(bool)), this, SLOT(loadFinished(bool)));
     connect(m_webView, SIGNAL(titleChanged(const QString &)), this, SIGNAL(titleChanged(const QString &)));
+
+    // Session Manager
+    connect(m_webView, SIGNAL(loadFinished(bool)), rApp->sessionManager(), SLOT(saveSession()));
 }
 
 
@@ -141,11 +146,13 @@ void WebTab::createWalletBar(const QString &key, const QUrl &url)
         m_walletBar = new WalletBar(this);
         m_walletBar.data()->onSaveFormData(key, url);
         qobject_cast<QVBoxLayout *>(layout())->insertWidget(0, m_walletBar.data());
+        m_walletBar.data()->animatedShow();
     }
     else
     {
         disconnect(wallet);
-        m_walletBar.data()->notifyUser();
+        qobject_cast<QVBoxLayout *>(layout())->insertWidget(0, m_walletBar.data());
+        m_walletBar.data()->animatedShow();
     }
 
     connect(m_walletBar.data(), SIGNAL(saveFormDataAccepted(const QString &)),
@@ -161,24 +168,19 @@ void WebTab::createPreviewSelectorBar(int index)
     {
         m_previewSelectorBar = new PreviewSelectorBar(index, this);
         qobject_cast<QVBoxLayout *>(layout())->insertWidget(0, m_previewSelectorBar.data());
+        m_previewSelectorBar.data()->animatedShow();
     }
     else
     {
         disconnect(m_previewSelectorBar.data());
         m_previewSelectorBar.data()->setIndex(index);
-        m_previewSelectorBar.data()->notifyUser();
+        m_previewSelectorBar.data()->animatedHide();
     }
 
     connect(page(),             SIGNAL(loadStarted()),      m_previewSelectorBar.data(), SLOT(loadProgress()), Qt::UniqueConnection);
     connect(page(),             SIGNAL(loadProgress(int)),  m_previewSelectorBar.data(), SLOT(loadProgress()), Qt::UniqueConnection);
     connect(page(),             SIGNAL(loadFinished(bool)), m_previewSelectorBar.data(), SLOT(loadFinished()), Qt::UniqueConnection);
-    connect(page()->mainFrame(), SIGNAL(urlChanged(QUrl)),  m_previewSelectorBar.data(), SLOT(verifyUrl()), Qt::UniqueConnection);
-}
-
-
-void WebTab::insertBar(NotificationBar *bar)
-{
-    qobject_cast<QVBoxLayout *>(layout())->insertWidget(0, bar);
+    connect(page()->mainFrame(), SIGNAL(urlChanged(QUrl)),  m_previewSelectorBar.data(), SLOT(verifyUrl()),    Qt::UniqueConnection);
 }
 
 
@@ -200,7 +202,7 @@ void WebTab::showRSSInfo(const QPoint &pos)
 
     QMap<KUrl, QString> map;
 
-    foreach(const QWebElement &el, col)
+    foreach(const QWebElement & el, col)
     {
         QString urlString;
         if (el.attribute("href").startsWith(QL1S("http")))
@@ -254,7 +256,7 @@ void WebTab::setPart(KParts::ReadOnlyPart *p, const KUrl &u)
 
 KUrl WebTab::extractOpensearchUrl(QWebElement e)
 {
-    QString href = e.attribute(QLatin1String("href"));
+    QString href = e.attribute(QL1S("href"));
     KUrl url = KUrl(href);
     if (!href.contains(":"))
     {
@@ -272,15 +274,15 @@ KUrl WebTab::extractOpensearchUrl(QWebElement e)
 
 bool WebTab::hasNewSearchEngine()
 {
-    QWebElement e = page()->mainFrame()->findFirstElement(QLatin1String("head >link[rel=\"search\"][ type=\"application/opensearchdescription+xml\"]"));
+    QWebElement e = page()->mainFrame()->findFirstElement(QL1S("head >link[rel=\"search\"][ type=\"application/opensearchdescription+xml\"]"));
     return !e.isNull() && !rApp->opensearchManager()->engineExists(extractOpensearchUrl(e));
 }
 
 
 void WebTab::showSearchEngine(const QPoint &pos)
 {
-    QWebElement e = page()->mainFrame()->findFirstElement(QLatin1String("head >link[rel=\"search\"][ type=\"application/opensearchdescription+xml\"]"));
-    QString title = e.attribute(QLatin1String("title"));
+    QWebElement e = page()->mainFrame()->findFirstElement(QL1S("head >link[rel=\"search\"][ type=\"application/opensearchdescription+xml\"]"));
+    QString title = e.attribute(QL1S("title"));
     if (!title.isEmpty())
     {
         WebShortcutWidget *widget = new WebShortcutWidget(window());
@@ -304,3 +306,16 @@ void WebTab::openSearchEngineAdded()
     disconnect(rApp->opensearchManager(), SIGNAL(openSearchEngineAdded(const QString &, const QString &, const QString &)),
                this, SLOT(openSearchEngineAdded()));
 }
+
+
+void WebTab::showMessageBar()
+{
+    MessageBar *msgBar = new MessageBar(i18n("It seems rekonq was not closed properly. Do you want "
+                                        "to restore the last saved session?")
+                                        , this);
+
+    qobject_cast<QVBoxLayout *>(layout())->insertWidget(0, msgBar);
+    msgBar->animatedShow();
+
+    connect(msgBar, SIGNAL(accepted()), rApp->sessionManager(), SLOT(restoreSession()));
+}
