@@ -77,7 +77,6 @@ public:
 
     KUrl url();
     void createPreviewSelectorBar(int index);
-    void insertBar(NotificationBar* bar);
 
     bool hasRSSInfo();
 
@@ -90,6 +89,8 @@ public:
     }
     void setPart(KParts::ReadOnlyPart *p, const KUrl &u);
 
+    void showMessageBar();
+
 private Q_SLOTS:
     void updateProgress(int progress);
     void loadFinished(bool);
