@@ -59,43 +59,28 @@
 
 #include <QtGui/QClipboard>
 #include <QtGui/QContextMenuEvent>
-#include <QtGui/QWindowsStyle>
 
 #include <QtWebKit/QWebFrame>
 #include <QtWebKit/QWebHistory>
 
 
 WebView::WebView(QWidget* parent)
-        : KWebView(parent, false)
-        , m_mousePos(QPoint(0, 0))
-        , m_autoScrollTimer(new QTimer(this))
-        , m_vScrollSpeed(0)
-        , m_hScrollSpeed(0)
-        , m_canEnableAutoScroll(true)
-        , m_isAutoScrollEnabled(false)
-        , m_smoothScrollTimer(new QTimer(this))
-        , m_smoothScrolling(false)
-        , m_dy(0)
-        , m_smoothScrollSteps(0)
+    : KWebView(parent, false)
+    , m_mousePos(QPoint(0, 0))
+    , m_autoScrollTimer(new QTimer(this))
+    , m_vScrollSpeed(0)
+    , m_hScrollSpeed(0)
+    , m_canEnableAutoScroll(true)
+    , m_isAutoScrollEnabled(false)
+    , m_autoScrollIndicator(QPixmap(KStandardDirs::locate("appdata" , "pics/autoscroll.png")))
+    , m_smoothScrollTimer(new QTimer(this))
+    , m_smoothScrolling(false)
+    , m_dy(0)
+    , m_smoothScrollSteps(0)
 {
     WebPage *page = new WebPage(this);
     setPage(page);
 
-    // // NOTE This is a lot hackish. We copied it from Arora, but using a "Windows Style"
-    // // seems really a pity to me. The problem is that using a KStyle everything seems "broken"
-    // // (at least with dark colors). So I think the code should be somthing like:
-    // KStyle s;
-    // setPalette( s.standardPalette() );
-    // // But it doesn't work :(
-    // // We'll see in next KDE releases...
-    QPalette p;
-    if (p.color(QPalette::ButtonText).lightness() > 50)   //if it is a dark theme
-    {
-        QWindowsStyle s;
-        p = s.standardPalette();
-        setPalette(p);
-    }
-
     // download system
     connect(this, SIGNAL(linkShiftClicked(const KUrl &)), page, SLOT(downloadUrl(const KUrl &)));
 
@@ -178,6 +163,11 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
         connect(a, SIGNAL(triggered(bool)), this, SLOT(openLinkInNewWindow()));
         menu.addAction(a);
 
+        a = new KAction(KIcon("bookmark-new"), i18n("&Bookmark this Link"), this);
+        a->setData(result.linkUrl());
+        connect(a, SIGNAL(triggered(bool)), this, SLOT(bookmarkLink()));
+        menu.addAction(a);
+
         menu.addSeparator();
         menu.addAction(pageAction(KWebPage::DownloadLinkToDisk));
         menu.addAction(pageAction(KWebPage::CopyLinkToClipboard));
@@ -239,9 +229,21 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
     // is content selected? Add SEARCH actions
     if (result.isContentSelected())
     {
+        //Default SearchEngine
+        KService::Ptr defaultEngine = SearchEngine::defaultEngine();
+        if (defaultEngine) // check if a default engine is set
+        {
+            a = new KAction(i18nc("Search selected text with the default search engine", "Search with %1", defaultEngine->name()), this);
+            a->setIcon(rApp->iconManager()->iconForUrl(SearchEngine::buildQuery(defaultEngine, "")));
+            a->setData(defaultEngine->entryPath());
+            connect(a, SIGNAL(triggered(bool)), this, SLOT(search()));
+            menu.addAction(a);
+        }
+
+        //All favourite ones
         KActionMenu *searchMenu = new KActionMenu(KIcon("edit-find"), i18n("Search with"), this);
 
-        foreach(KService::Ptr engine, SearchEngine::favorites())
+        foreach(const KService::Ptr & engine, SearchEngine::favorites())
         {
             a = new KAction(engine->name(), this);
             a->setIcon(rApp->iconManager()->iconForUrl(SearchEngine::buildQuery(engine, "")));
@@ -261,8 +263,7 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
 
         menu.addSeparator();
 
-        if (ReKonfig::showDeveloperTools())
-            menu.addAction(inspectAction);
+        menu.addAction(inspectAction);
         // TODO Add translate, show translation
     }
 
@@ -287,25 +288,9 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
 
         menu.addSeparator();
 
-        if (ReKonfig::showDeveloperTools())
-            menu.addAction(inspectAction);
+        menu.addAction(inspectAction);
     }
 
-// TODO: REMOVE ME!
-// These actions are here just to NOT remove strings until stable release.
-// Just remove the lines here after that.
-if (false)
-{
-    QString truncatedURL;
-
-    //open selected text url in a new tab
-    a = new KAction(KIcon("tab-new"), i18n("Open '%1' in New Tab", truncatedURL), this);
-
-    //open selected text url in a new window
-    a = new KAction(KIcon("window-new"), i18n("Open '%1' in New Window", truncatedURL), this);
-}
-// TODO: ----------- END REMOVE
-
     // page actions
     if (!result.isContentSelected() && result.linkUrl().isEmpty())
     {
@@ -360,11 +345,8 @@ if (false)
                 menu.addAction(a);
             }
 
-            if (ReKonfig::showDeveloperTools())
-            {
-                menu.addAction(mainwindow->actionByName("page_source"));
-                menu.addAction(inspectAction);
-            }
+            menu.addAction(mainwindow->actionByName("page_source"));
+            menu.addAction(inspectAction);
 
             a = rApp->bookmarkProvider()->actionByName("rekonq_add_bookmark");
             menu.addAction(a);
@@ -385,11 +367,11 @@ void WebView::mousePressEvent(QMouseEvent *event)
 {
     if (m_isAutoScrollEnabled)
     {
-        setCursor(Qt::ArrowCursor);
         m_vScrollSpeed = 0;
         m_hScrollSpeed = 0;
         m_autoScrollTimer->stop();
         m_isAutoScrollEnabled = false;
+        update();
         return;
     }
 
@@ -415,9 +397,26 @@ void WebView::mousePressEvent(QMouseEvent *event)
             if (!page()->currentFrame()->scrollBarGeometry(Qt::Horizontal).isNull()
                     || !page()->currentFrame()->scrollBarGeometry(Qt::Vertical).isNull())
             {
-                setCursor(KIcon("transform-move").pixmap(32));
                 m_clickPos = event->pos();
                 m_isAutoScrollEnabled = true;
+                update();
+            }
+        }
+
+        if (!ReKonfig::autoScroll())
+        {
+            const QString clipboardContent = rApp->clipboard()->text();
+
+            if (clipboardContent.isEmpty())
+                break;
+
+            if (QUrl::fromUserInput(clipboardContent).isValid())
+                loadUrl(clipboardContent, Rekonq::CurrentTab);
+            else // Search with default Engine
+            {
+                KService::Ptr defaultEngine = SearchEngine::defaultEngine();
+                if (defaultEngine) // check if a default engine is set
+                    loadUrl(KUrl(SearchEngine::buildQuery(defaultEngine, clipboardContent)), Rekonq::CurrentTab);
             }
         }
         break;
@@ -461,15 +460,6 @@ void WebView::mouseMoveEvent(QMouseEvent *event)
 }
 
 
-void WebView::enterEvent(QEvent *event)
-{
-    if (m_isAutoScrollEnabled)
-        setCursor(KIcon("transform-move").pixmap(32));
-
-    KWebView::enterEvent(event);
-}
-
-
 void WebView::dropEvent(QDropEvent *event)
 {
     bool isEditable = page()->frameAt(event->pos())->hitTestContent(event->pos()).isContentEditable();
@@ -488,7 +478,7 @@ void WebView::dropEvent(QDropEvent *event)
     }
     else if (event->mimeData()->hasUrls() && event->source() != this && !isEditable) //dropped links
     {
-        Q_FOREACH (const QUrl &url, event->mimeData()->urls())
+        Q_FOREACH(const QUrl & url, event->mimeData()->urls())
         {
             emit loadUrl(url, Rekonq::NewFocusedTab);
         }
@@ -506,6 +496,24 @@ void WebView::dropEvent(QDropEvent *event)
     }
 }
 
+
+void WebView::paintEvent(QPaintEvent* event)
+{
+    KWebView::paintEvent(event);
+
+    if (m_isAutoScrollEnabled)
+    {
+        QPoint centeredPoint = m_clickPos;
+        centeredPoint.setX(centeredPoint.x() - m_autoScrollIndicator.width() / 2);
+        centeredPoint.setY(centeredPoint.y() - m_autoScrollIndicator.height() / 2);
+
+        QPainter painter(this);
+        painter.setOpacity(0.8);
+        painter.drawPixmap(centeredPoint, m_autoScrollIndicator);
+    }
+}
+
+
 void WebView::search()
 {
     KAction *a = qobject_cast<KAction*>(sender());
@@ -573,6 +581,16 @@ void WebView::openLinkInNewTab()
 }
 
 
+void WebView::bookmarkLink()
+{
+    KAction *a = qobject_cast<KAction*>(sender());
+    KUrl url(a->data().toUrl());
+
+    rApp->bookmarkProvider()->rootGroup().addBookmark(url.prettyUrl(), url);
+    rApp->bookmarkProvider()->bookmarkManager()->emitChanged();
+}
+
+
 void WebView::keyPressEvent(QKeyEvent *event)
 {
     if (event->modifiers() == Qt::ControlModifier)
@@ -642,6 +660,35 @@ void WebView::keyPressEvent(QKeyEvent *event)
         }
     }
 
+    // vi-like navigation
+    if (ReKonfig::enableViShortcuts())
+    {
+        const QString tagName = page()->mainFrame()->evaluateJavaScript("document.activeElement.tagName").toString();
+        if (tagName != QL1S("INPUT") && tagName != QL1S("TEXTAREA") && event->modifiers() == Qt::NoModifier)
+        {
+            switch (event->key())
+            {
+            case Qt::Key_J:
+                event->accept();
+                event = new QKeyEvent(QEvent::KeyPress, Qt::Key_Down, Qt::NoModifier);
+                break;
+            case Qt::Key_K:
+                event->accept();
+                event = new QKeyEvent(QEvent::KeyPress, Qt::Key_Up, Qt::NoModifier);
+                break;
+            case Qt::Key_L:
+                event->accept();
+                event = new QKeyEvent(QEvent::KeyPress, Qt::Key_Right, Qt::NoModifier);
+                break;
+            case Qt::Key_H:
+                event->accept();
+                event = new QKeyEvent(QEvent::KeyPress, Qt::Key_Left, Qt::NoModifier);
+                break;
+            default:
+                break;
+            }
+        }
+    }
     KWebView::keyPressEvent(event);
 }
 
